@@ -35,6 +35,11 @@ namespace Clasp {
 #if !defined(SIGUSR2)
 #define SIGUSR2 SIGTERM
 #endif
+#if !defined(COMPETITION_MODE) || COMPETITION_MODE == 0
+#define CLASP_APP_RETURN(x) return x
+#else
+#define CLASP_APP_RETURN(x) _exit(x)
+#endif
 namespace {
 std::ostream& operator << (std::ostream& os, Literal l) {
 	if (l.sign()) os << '-';
@@ -190,21 +195,22 @@ int Application::run(int argc, char** argv) {
 	if (!parse(argc, argv, "clasp", parsePositional)) {
 		// command-line error
 		ERROR_OUT("*** clasp", messages.error.c_str());
-		return S_ERROR;
+		CLASP_APP_RETURN(S_ERROR);
 	}
 	if (help || version) {
-		return EXIT_SUCCESS;
+		CLASP_APP_RETURN(EXIT_SUCCESS);
 	}
 	if (claspre_.listFeatures) {
 		claspre_.printFeatures();
-		return EXIT_SUCCESS;
+		CLASP_APP_RETURN(EXIT_SUCCESS);
 	}
-	if (config_.thread.genTemplate) {
+	if (cmdOpts_.genTemplate) {
 		printTemplate();
-		return EXIT_SUCCESS;
+		CLASP_APP_RETURN(EXIT_SUCCESS);
 	}	
 	installSigHandlers();
 	int retStatus = S_UNKNOWN;
+	ClaspFacade clasp;
 	try {
 		StreamInput input(getStream(), detectFormat(getStream()));
 		cmdOpts_.applyDefaults(input.format());
@@ -217,7 +223,7 @@ int Application::run(int argc, char** argv) {
 		if (cmdOpts_.basic.stats > 1) {
 			config_.master()->solver().stats.enableJumpStats();
 		}
-		ClaspFacade clasp; facade_ = &clasp;
+		facade_ = &clasp;
 		cpuTotalTime_.start();
 		clasp.solve(input, config_, this);
 		cpuTotalTime_.stop();
@@ -227,16 +233,12 @@ int Application::run(int argc, char** argv) {
 		if      (clasp.result() == ClaspFacade::result_unsat) retStatus = S_UNSATISFIABLE;
 		else if (clasp.result() == ClaspFacade::result_sat)   retStatus = S_SATISFIABLE;
 		else                                                  retStatus = S_UNKNOWN;
+		fflush(stdout);
+		fflush(stderr);
 	}
 	catch (const std::bad_alloc&  ) { retStatus = exception(S_MEMORY, "std::bad_alloc"); }
 	catch (const std::exception& e) { retStatus = exception(S_ERROR, e.what()); }
-#ifndef COMPETITION_MODE
-	return retStatus;
-#else
-	fflush(stdout);
-	fflush(stderr);
-	_exit(retStatus);
-#endif
+	CLASP_APP_RETURN(retStatus);
 }
 
 int Application::exception(int status, const char* what) {
