@@ -24,9 +24,14 @@
 #include <sstream>
 #include <fstream>
 #include <algorithm>
-
 using namespace ProgramOptions;
 using namespace std;
+
+#ifdef _DEBUG
+#define STATE_DEFAULTED Value::value_unassigned 
+#else
+#define STATE_DEFAULTED Value::value_defaulted
+#endif
 namespace Clasp { 
 /////////////////////////////////////////////////////////////////////////////////////////
 // Parseing & Mapping of options
@@ -52,6 +57,7 @@ ProgramOptions::StringSlice parseValue(const ProgramOptions::StringSlice& in, Sc
 	if (in.size() > 0) {
 		if      (parseValue(in, v.second, 0).ok()) { ok = true; }
 		else if (parseValue(in, v, 0).ok())        {
+			v.first = toLower(v.first);
 			if      (v.first == "no")                      { ok = v.second.empty(); }
 			else if (v.first == "+" || v.first == "arith") { ok = v.second.size() >= 2; v.first = "+"; }
 			else if (v.first == "*" || v.first == "geom")  { ok = v.second.size() >= 2; }
@@ -128,43 +134,10 @@ bool BasicOptions::mapIfs(const std::string& s, char& ifs) {
 // Clasp specific mode options
 /////////////////////////////////////////////////////////////////////////////////////////
 void GeneralOptions::initOptions(ProgramOptions::OptionContext& root) {
-#ifndef DISABLE_MULTI_THREADING
-	OptionGroup threadOps("Clasp - Thread Options");
-	ThreadOptions* thread = &config->thread;
-	threadOps.addOptions() 
-		("threads,t", notify(this, &GeneralOptions::mapThreadOpts)->defaultsTo("1")->state(Value::value_defaulted)->arg("<n>"), 
-			"Set number of threads to use\n"
-			"      Default: %D / Valid: 0 <= %A < 64")
-		("portfolio,p", storeTo(thread->portfolio)->arg("<file>"), 
-		 "Use portfolio to configure threads\n"
-		 "      Valid: %A or 'default'")
-		("create-template,g" , flag(thread->genTemplate), "Print a template portfolio and exit\n")
-		("force-gp", flag(thread->forceGP), "Force guiding path scheme even if portfolio is used")
-		("global-restarts", notify(this, &GeneralOptions::mapThreadOpts)->implicit("5,100,1.5"),
-		 "Configure global restart policy (0 if not given)\n"
-		 "      Valid: <n>[,strat>] / Implicit: %I\n"
-		 "        <n> : Maximal number of global restarts (0=disable)\n"
-		 "     <strat>: Restart strategy (see --restarts)\n")
-		("copy-problem", flag(thread->distribute.copyProblem), "Copy (instead of share) problem between threads")
-		("distribute", notify(this, &GeneralOptions::mapThreadOpts)->defaultsTo("no")->state(Value::value_defaulted),
-			"Configure nogood distribution\n"
-			"      Default: %D\n"
-			"      Valid:   <type>[,<lbd>]\n"
-			"        <type> : Nogoods to distribute (no, all, short, conflict, loop)\n"
-			"        <lbd>  : Distribute only if lbd <= <lbd>")
-		("integrate", notify(this, &GeneralOptions::mapThreadOpts)->defaultsTo("gp,1024")->state(Value::value_defaulted),
-			"Configure nogood integration\n"
-			"      Default: %D\n"
-			"      Valid:   <pick>[,<grace>]\n"
-			"        <pick>  : Selector to apply (all, unsat, gp, heuristic)\n"
-			"        <grace> : Keep at least last <grace> shared nogoods")
-	;
-	root.add(threadOps);
-#endif
 	GlobalOptions* global = config;
 	OptionGroup general("Clasp - General Options");
 	general.addOptions()
-		("enum-mode", storeTo(global->enumerate.mode, &parseEnumMode)->defaultsTo("auto")->state(Value::value_defaulted), 
+		("enum-mode", storeTo(global->enumerate.mode, &parseEnumMode)->defaultsTo("auto")->state(STATE_DEFAULTED), 
 			"Configure enumeration algorithm\n"
 			"      Default: %D\n"
 			"      Valid:   auto, bt, record, brave, cautious\n"
@@ -204,7 +177,7 @@ void GeneralOptions::initOptions(ProgramOptions::OptionContext& root) {
 		("backprop",flag(global->eq.backprop), "Enable backpropagation in ASP-preprocessing")
 		("trans-ext", storeTo(global->eq.erMode, &mapEnum<extRules>),
 			"Configure handling of Lparse-like extended rules\n"
-			"      Valid:   all, choice, card, weight, integ, dynamic, no (0)\n"
+			"      Valid:   all, choice, card, weight, integ, dynamic, no\n"
 			"        all    : Transform all extended rules to basic rules\n"
 			"        choice : Transform choice rules, but keep cardinality and weight rules\n"
 			"        card   : Transform cardinality rules, but keep choice and weight rules\n"
@@ -223,6 +196,7 @@ void GeneralOptions::initOptions(ProgramOptions::OptionContext& root) {
 		("solution-recording" , storeTo(global->enumerate.mode, &parseEnumMode)->flag()->implicit("record"), "Alias for --enum-mode=%I")
 		("project-opt", storeTo(global->enumerate.projectOpts), "Additional options for projection as octal digit\n")
 		("eq-dfs"     , flag(global->eq.dfOrder)    , "Enable df-order in eq-preprocessing\n")
+		("learn-explicit", flag(this, GeneralOptions::mapLearnExplicit), "Don't use Short Implication Graph for learning\n")
 	;
 	root.add(hidden);
 }
@@ -247,22 +221,25 @@ bool GeneralOptions::parseEnumMode(const std::string& s, GlobalOptions::EnumMode
 	}
 	return false;
 }
+bool GeneralOptions::mapLearnExplicit(GeneralOptions* this_, const std::string&, const bool* v) {
+	this_->config->ctx.enableUpdateShortImplications(!*v);
+	return false;
+}
 bool GeneralOptions::mapMyVec(GeneralOptions* this_, const std::string& n, const std::vector<int>* v) {
 	GlobalOptions* global = this_->config;
 	assert(v == &this_->myVec);
 	if (n == "sat-prepro") {
 		if (!v->empty() && v->at(0) != 0) {
 			SatElite::SatElite* pre = new SatElite::SatElite();
-			pre->options.maxIters = v->size()>0 ? v->at(0) : -1;
-			pre->options.maxOcc   = v->size()>1 ? v->at(1) : -1;
-			pre->options.maxTime  = v->size()>2 ? v->at(2) : -1;
-			pre->options.maxFrozen= v->size()>3 && v->at(3) > 0 ? (v->at(3)/100.0) : 1.0;
+			pre->options.maxIters   = v->size()>0 ? v->at(0) : -1;
+			pre->options.maxOcc     = v->size()>1 ? v->at(1) : -1;
+			pre->options.maxTime    = v->size()>2 ? v->at(2) : -1;
+			pre->options.maxFrozen  = v->size()>3 && v->at(3) > 0 ? (v->at(3)/100.0) : 1.0;
 			if (v->size() > 4 && v->at(4) >= 0) {
-				pre->options.bce    = (uint32)v->at(4);
+				pre->options.bce      = (uint32)v->at(4);
 			}
 			global->ctx.satPrepro.reset(pre);
 		}
-		this_->satPreDefault = false;
 	}
 	else if (n == "opt-all" || !global->opt.all) {
 		global->opt.vals.assign(v->begin(), v->end());
@@ -272,7 +249,297 @@ bool GeneralOptions::mapMyVec(GeneralOptions* this_, const std::string& n, const
 	return true; // keep vector on return
 }
 
-bool GeneralOptions::mapThreadOpts(GeneralOptions* this_, const std::string& key, const std::string& value) {
+bool GeneralOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Messages& m) {
+	if (vm.count("opt-value") && vm.count("opt-all")) {
+		m.warning.push_back("'opt-all' and 'opt-value' are mutually exclusive!");
+	}
+	if (config->eq.noSCC && vm.count("eq") == 0) {
+		config->eq.noEq();
+	}
+	return true;
+}
+/////////////////////////////////////////////////////////////////////////////////////////
+// Clasp specific search options
+/////////////////////////////////////////////////////////////////////////////////////////
+SearchOptions::SearchOptions(LocalOptions* o) : local(o) {}
+SolverStrategies* SearchOptions::solverOpts() const { return &local->solver().strategies(); }
+void SearchOptions::initOptions(ProgramOptions::OptionContext& root) {
+	SolverStrategies*  solverOpts = this->solverOpts();
+	OptionGroup search("Clasp - Search Options");
+	search.addOptions()
+		("no-lookback"   , notify(this, &SearchOptions::mapCustom)->flag(), "Disable all lookback strategies\n")
+		("lookahead"     , storeTo(local->heuristic().lookahead, &mapEnum<lookTypes>)->implicit("atom"),
+			"Configure failed-literal detection\n"
+			"      Default: no (atom, if --nolookback)\n"
+			"      Valid:   atom, body, hybrid, no / Implicit: %I\n"
+			"        atom  : Apply failed-literal detection to atoms\n"
+			"        body  : Apply failed-literal detection to bodies\n"
+			"        hybrid: Apply Nomore++-like failed-literal detection\n"
+			"        no    : Do not apply failed-literal detection")
+		("initial-lookahead", storeTo(local->heuristic().lookaheadNum)->arg("<n>"), "Restrict failed-literal detection to %A decisions\n")
+
+		("heuristic", storeTo(local->heuristic().name, &SearchOptions::parseHeuristic)->defaultsTo("Berkmin")->state(STATE_DEFAULTED), 
+			"Configure decision heuristic\n"
+			"      Default: %D (Unit, if --no-lookback)\n"
+			"      Valid:   Berkmin, Vmtf, Vsids, Unit, None\n"
+			"        Berkmin: Apply BerkMin-like heuristic\n"
+			"        Vmtf   : Apply Siege-like heuristic\n"
+			"        Vsids  : Apply Chaff-like heuristic\n"
+			"        Unit   : Apply Smodels-like heuristic\n"
+			"        None   : Select the first free variable")
+		("opt-heuristic" , storeTo(local->solver().strategies().heuOpts)->implicit("1")->arg("<n>"), 
+		 "Configure optimize heuristic\n"
+		  "      Valid: 0 <= <n> <= 3 / Implicit: %I \n"
+		  "         1: Apply sign heuristic\n"
+		  "         2: Apply model heuristic\n"
+		  "         3: Apply sign and model heuristic")
+		("rand-freq", notify(this, &SearchOptions::mapRandOpts)->arg("<p>"), 
+			"Make random decisions with probability %A\n"
+			"      Valid:   [0.0...1.0]\n")
+
+		("rand-prob", notify(this, &SearchOptions::mapRandOpts)->implicit("50,20")->arg("<n1,n2>"),
+			"Configure random probing\n"
+			"      Valid:   no (0), <n1,n2> (<n1> >= 0, <n2> > 0) / Implicit: %I\n"
+			"        no     : Do not run random probing\n"
+			"        <n1,n2>: Run <n1> random passes up to at most <n2> conflicts each\n")
+		("rand-watches", storeTo(local->solver().strategies().randomWatches)->implicit("true")->defaultsTo("true"),
+			"Configure watched literal initialization\n"
+			"      Default: %D / Implicit: %I\n"
+			"      Valid:   true (yes, 1), false (no, 0)\n"
+			"        true : Randomly determine watched literals\n"
+			"        false: Watch least watched literals in a nogood")
+		("seed"    , storeNotify(aNumber, this, &SearchOptions::mapNumber)->arg("<n>"),    "Set random number generator's seed to %A\n")
+		("search-limit", notify(this, &SearchOptions::mapCustom)->arg("<n,m>"), "Stop search after <n> conflicts or <m> restarts\n")
+	;
+	
+	OptionGroup lookback("Clasp - Lookback Options");
+	lookback.addOptions()
+		("restarts,r", storeTo(local->solve.restart.sched, &parseSchedule)->defaultsTo("100,1.5")->state(STATE_DEFAULTED),
+			"Configure restart policy\n"
+			"      Default: %D\n"
+			"      Valid:   [type,]<n1[,n2,n3]> (<n1> >= 0, <n2>,<n3> > 0), no\n"
+			"      [L,]<n1>          : Run Luby et al.'s sequence with unit length <n1>\n"
+			"      [*,]<n1>,<n2>     : Run geometric sequence of <n1>*(<n2>^i) conflicts\n"
+			"      +,<n1>,<n2>       : Run arithmetic sequence of <n1>+(<n2>*i) conflicts\n"
+			"      [+,]<n1>,<n2>,<n3>: Run Biere's inner-outer sequence (<n3>=outer)\n"
+			"          <n1> = 0, no  : Disable restarts")
+		("local-restarts"  , flag(local->solve.restart.local), "Enable Ryvchin et al.'s local restarts")
+		("bounded-restarts", flag(local->solve.restart.bounded), "Enable (bounded) restarts during model enumeration")
+		("reset-restarts",   flag(local->solve.restart.resetOnModel), "Reset restart strategy during model enumeration")
+		("save-progress"   , storeTo(solverOpts->saveProgress)->implicit("1")->arg("<n>"), "Enable RSat-like progress saving on backjumps > %A")
+
+		("shuffle,s", notify(this, &SearchOptions::mapCustom)->arg("<n1,n2>"),
+			"Configure shuffling after restarts\n"
+			"      Valid:   <n1,n2> (<n1> >= 0, <n2> >= 0)\n"
+			"        <n1> > 0: Shuffle problem after <n1> and re-shuffle every <n2> restarts\n"
+			"        <n1> = 0: Do not shuffle problem after restarts\n"
+			"        <n2> = 0: Do not re-shuffle problem\n")
+
+		("deletion,d", notify(this, &SearchOptions::mapCustom)->defaultsTo("3.0,1.1,3.0")->state(STATE_DEFAULTED), 
+			"Configure size of learnt nogood database\n"
+			"      Default: %D\n"
+			"      Valid:   <n1[,n2,n3]> (<n3> >= <n1> >= 0, <n2> >= 1.0), no\n"
+			"        <n1,n2,n3>: Store at most min(P/<n1>*(<n2>^i),P*<n3>) learnt nogoods,\n"
+			"                    P and i being initial problem size and number of restarts\n"
+			"        no        : Do not restrict learnt db size")
+		("dsched", storeTo(local->solve.reduce.sched, &parseSchedule), "Configure secondary deletion policy (see restarts)")
+		("dglue", storeTo(solverOpts->reduceGlue), "Don't delete nogoods with lbd <= x")
+		("update-lbd", flag(solverOpts->updateLbd), "Dynamically update lbds of learnt nogoods")
+		("reduce-on-restart", flag(local->solve.reduce.reduceOnRestart), "Delete some learnt nogoods after every restart\n")
+		("estimate", flag(local->solve.reduce.estimate), "Use estimated problem complexity to init learnt db")
+		("dinit",notify(this, &SearchOptions::mapCustom)->arg("<min,max>"), "Limit initial learnt db size to range [min,max]\n")
+
+		("strengthen", storeTo(solverOpts->cflMinAntes, &mapEnum<anteTypes>)->defaultsTo("all")->state(STATE_DEFAULTED),
+			"Configure conflict nogood strengthening\n"
+			"      Default: %D\n"
+			"      Valid:   bin, tern, all, no\n"
+			"        bin : Check only binary antecedents for self-subsumption\n"
+			"        tern: Check binary and ternary antecedents for self-subsumption\n"
+			"        all : Check all antecedents for self-subsumption\n"
+			"        no  : Do not check antecedents for self-subsumption")
+		("recursive-str", flag(solverOpts->strengthenRecursive), "Enable MiniSAT-like conflict nogood strengthening")
+		("otfs",storeTo(solverOpts->otfs)->implicit("1"), "Enable on-the-fly subsumption (1=partial, 2=full)")
+		("reverse-arcs",storeTo(solverOpts->reverseArcs)->implicit("1"), "Enable ManySAT-like reverse-arcs\n")
+		
+		("loops", storeTo(local->loopRep, &mapEnum<loopTypes>)->defaultsTo("common")->state(STATE_DEFAULTED),
+			"Configure representation/learning of loop formulas\n"
+			"      Default: %D\n"
+			"      Valid:   common, distinct, shared, no\n"
+			"        common  : Create loop nogoods for atoms in an unfounded set\n"
+			"        distinct: Create distinct loop nogood for each atom in an unfounded set\n"
+			"        shared  : Create loop formula for a whole unfounded set\n"
+			"        no      : Do not learn loop formulas")
+		("loops-in-heu", storeTo(local->heuristic().loops), "Consider loop nogoods in heuristics\n")
+
+		("contraction", storeNotify(aNumber, this, &SearchOptions::mapNumber)->arg("<n>")->defaultsTo("250")->state(STATE_DEFAULTED),
+			"Configure temporary contraction of learnt nogoods\n"
+			"      Default: %D\n"
+			"      Valid:\n"
+			"        0  : Do not contract learnt nogoods\n"
+			"        > 0: Contract learnt nogoods containing more than %A literals\n")
+	;
+	ProgramOptions::OptionGroup hidden("Hidden Options");
+	hidden.hidden(true);
+	hidden.addOptions()
+		("berk-max", storeTo(local->heuristic().extra.berkMax), "Consider at most <n> nogoods in Berkmin heuristic")
+		("berk-moms", flag(local->heuristic().berkMoms), "Enable/Disable MOMs in Berkmin")
+		("berk-huang",flag(local->heuristic().berkHuang), "Enable/Disable Huang-scoring in Berkmin")
+		("berk-once",flag(local->heuristic().berkOnce), "Score sets (instead of multisets) in Berkmin")
+		("vmtf-mtf",storeTo(local->heuristic().extra.vmtfMtf), "In Vmtf move <n> conflict-literals to the front")
+		("nant",flag(local->heuristic().nant), "In Unit count only atoms in NAnt(P)\n")
+		("dfrac", storeNotify(aNumber, this, &SearchOptions::mapNumber), "Set fraction of nogoods to delete on reduction")
+		("dalgo", storeTo(solverOpts->reduceAlgo), "Use {0=basic, 1=sort-in-place, 2=sorted} deletion")
+		("dscore", storeTo(solverOpts->reduceScore), "Use {0=activity, 1=lbd, 2=combined} nogood scores")
+		("cir", storeNotify(aNumber, this, &SearchOptions::mapNumber), "Enable CIR with interval %A (0=disable)")
+		("cir-bump", storeTo(local->solve.restart.cirBump), "Set CIR bump factor to %A")
+	;
+	root.add(search);
+	root.add(lookback);
+	root.add(hidden);
+}
+const EnumMap SearchOptions::loopTypes[] = {
+	{"common", DefaultUnfoundedCheck::common_reason}, {"shared", DefaultUnfoundedCheck::shared_reason},
+	{"distinct", DefaultUnfoundedCheck::distinct_reason}, {"no", DefaultUnfoundedCheck::only_reason},
+	{0,0}
+};
+const EnumMap SearchOptions::lookTypes[] = {
+	{"atom", Lookahead::atom_lookahead}, {"no", Lookahead::no_lookahead},
+	{"body", Lookahead::body_lookahead}, {"hybrid", Lookahead::hybrid_lookahead},
+	{0,0}
+};
+const EnumMap SearchOptions::anteTypes[] = {
+	{"all", SolverStrategies::all_antes}, {"no", SolverStrategies::no_antes},
+	{"bin", SolverStrategies::binary_antes}, {"tern", SolverStrategies::binary_ternary_antes},
+	{0,0}
+};
+
+bool SearchOptions::parseHeuristic(const std::string& s, std::string& out) {
+	std::string temp = toLower(s);
+	if      (temp == "berkmin")   { out = temp; return true; }
+	else if (temp == "vmtf")      { out = temp; return true; }
+	else if (temp == "vsids")     { out = temp; return true; }
+	else if (temp == "unit")      { out = temp; return true; }
+	else if (temp == "none")      { out = temp; return true; }
+	return false;
+}
+
+bool SearchOptions::mapRandOpts(SearchOptions* this_, const std::string& opt, const std::string& value) {
+	std::pair<double, double> v(0, 0);
+	if (value == "no" || parse(value, v)) {
+		return (opt == "rand-freq" && this_->local->solve.setRandomProbability(v.first))
+		  ||   (opt == "rand-prob" && this_->local->solve.setRandomizeParams((uint32)v.first, (uint32)v.second));
+	}
+	return false;
+}
+
+bool SearchOptions::parseSchedule(const std::string& s, ScheduleStrategy& sched) {
+	return parseValue(StringSlice(s), sched, 0).ok();
+}
+
+bool SearchOptions::mapNumber(SearchOptions* this_, const std::string& opt, const double* v) {
+	if      (opt == "seed")        { this_->solverOpts()->rng.srand(static_cast<uint32>(*v)); }
+	else if (opt == "contraction") { this_->solverOpts()->setCompressionStrategy(static_cast<uint32>(*v)); }
+	else if (opt == "dfrac")       { this_->local->solve.reduce.setReduceFraction(*v); }
+	else if (opt == "cir")         { this_->local->solve.restart.cir = (uint8)*v; }
+	this_->aNumber = 0.0;
+	return true;
+}
+
+bool SearchOptions::mapCustom(SearchOptions* this_, const std::string& n, const std::string& value) {
+	bool ok = false;
+	if (n == "no-lookback") {
+		bool b;
+		ok    = FlagStr::store_true(value, b);
+		if (ok && b) this_->solverOpts()->search = Clasp::SolverStrategies::no_learning;
+	}
+	else if (n == "deletion") {
+		std::vector<double> v;
+		if (value == "no" || parse(value, v)) {
+			ok = true;
+			if (v.size()==1) { v.push_back(1.1); }
+			if (v.size()==2) { v.push_back(3.0); }
+			if (v.empty())   { this_->local->solve.reduce.disable = true; }
+			else             { this_->local->solve.reduce.setStrategy(v[0], v[1], v[2]); }
+		}
+	}
+	else {
+		std::pair<int, int> p(-1, -1);
+		if ((ok = parse(value, p))) {
+			if (n == "dinit") {
+				this_->local->solve.reduce.setInit((uint32)p.first, (uint32)p.second);
+			}
+			else if (n == "shuffle") {
+				this_->local->solve.setShuffleParams(p.first, p.second);
+			}
+			else if (n == "search-limit") {
+				this_->local->solve.limits = SolveLimits(static_cast<uint64>(p.first), static_cast<uint64>(p.second));
+			}
+		}
+	}
+	return ok;
+}
+
+bool SearchOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Messages& m) {
+	if (solverOpts()->search == Clasp::SolverStrategies::no_learning) {	
+		if (vm.count("heuristic") == 0) { local->heuristic().name = "unit"; }
+		if (vm.count("lookahead") == 0) { local->heuristic().lookahead = Lookahead::atom_lookahead; }
+		bool warn = local->solve.restart.local || local->solve.restart.bounded || local->solve.reduce.reduceOnRestart;
+		if (warn || vm.count("restarts") || vm.count("deletion") || vm.count("rand-prob") || vm.count("shuffle")) {
+			m.warning.push_back("lookback-options ignored because lookback strategy is not used!");     
+		}
+	}
+	return true;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////
+// clasp option validation
+/////////////////////////////////////////////////////////////////////////////////////////
+void ClaspOptions::initOptions(ProgramOptions::OptionContext& root, ClaspConfig& config) {
+	satPreDefault = true;
+	genTemplate   = false;
+	this->config  = &config;
+	initThreadOptions(root);
+	mode.reset(new GeneralOptions(&config));
+	search.reset(new SearchOptions(config.master()));
+	mode->initOptions(root);
+	search->initOptions(root);
+	basic.initOptions(root);
+}
+#ifndef DISABLE_MULTI_THREADING
+void ClaspOptions::initThreadOptions(ProgramOptions::OptionContext& root) {
+	OptionGroup threadOps("Clasp - Thread Options");
+	ThreadOptions* thread = &config->thread;
+	threadOps.addOptions() 
+		("threads,t", notify(this, &ClaspOptions::mapThreadOpts)->defaultsTo("1")->state(STATE_DEFAULTED)->arg("<n>"), 
+			"Set number of threads to use\n"
+			"      Default: %D / Valid: 0 <= %A < 64")
+		("portfolio,p", storeTo(portfolio)->arg("<file>"), 
+		 "Use portfolio to configure threads\n"
+		 "      Valid: %A or 'default'")
+		("create-template,g" , flag(genTemplate), "Print a template portfolio and exit\n")
+		("force-gp", flag(thread->forceGP), "Force guiding path scheme even if portfolio is used")
+		("global-restarts", notify(this, &ClaspOptions::mapThreadOpts)->implicit("5,100,1.5"),
+		 "Configure global restart policy (0 if not given)\n"
+		 "      Valid: <n>[,strat>] / Implicit: %I\n"
+		 "        <n> : Maximal number of global restarts (0=disable)\n"
+		 "     <strat>: Restart strategy (see --restarts)\n")
+		("copy-problem", flag(thread->distribute.copyProblem), "Copy (instead of share) problem between threads")
+		("distribute", notify(this, &ClaspOptions::mapThreadOpts)->defaultsTo("no")->state(STATE_DEFAULTED),
+			"Configure nogood distribution\n"
+			"      Default: %D\n"
+			"      Valid:   <type>[,<lbd>]\n"
+			"        <type> : Nogoods to distribute (no, all, short, conflict, loop)\n"
+			"        <lbd>  : Distribute only if lbd <= <lbd>")
+		("integrate", notify(this, &ClaspOptions::mapThreadOpts)->defaultsTo("gp,1024")->state(STATE_DEFAULTED),
+			"Configure nogood integration\n"
+			"      Default: %D\n"
+			"      Valid:   <pick>[,<grace>]\n"
+			"        <pick>  : Selector to apply (all, unsat, gp, heuristic)\n"
+			"        <grace> : Keep at least last <grace> shared nogoods")
+	;
+	root.add(threadOps);
+}
+bool ClaspOptions::mapThreadOpts(ClaspOptions* this_, const std::string& key, const std::string& value) {
 	ThreadOptions* thread             = &this_->config->thread;
 	ThreadOptions::Distribution& opts = thread->distribute;
 	if (key == "threads") {
@@ -315,275 +582,27 @@ bool GeneralOptions::mapThreadOpts(GeneralOptions* this_, const std::string& key
 	}
 	return false; // value is not valid
 }
-
-bool GeneralOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Messages& m) {
-	if (vm.count("opt-value") && vm.count("opt-all")) {
-		m.warning.push_back("'opt-all' and 'opt-value' are mutually exclusive!");
-	}
-	if (config->eq.noSCC && vm.count("eq") == 0) {
-		config->eq.noEq();
-	}
-	return true;
-}
-/////////////////////////////////////////////////////////////////////////////////////////
-// Clasp specific search options
-/////////////////////////////////////////////////////////////////////////////////////////
-void SearchOptions::initOptions(ProgramOptions::OptionContext& root) {
-	OptionGroup search("Clasp - Search Options");
-	search.addOptions()
-		("no-lookback"   , notify(this, &SearchOptions::mapCustom)->flag(), "Disable all lookback strategies\n")
-		("lookahead"     , storeTo(local->heuristic().lookahead, &mapEnum<lookTypes>)->implicit("atom"),
-			"Configure failed-literal detection\n"
-			"      Default: no (atom, if --nolookback)\n"
-			"      Valid:   atom, body, hybrid, no (0) / Implicit: %I\n"
-			"        atom  : Apply failed-literal detection to atoms\n"
-			"        body  : Apply failed-literal detection to bodies\n"
-			"        hybrid: Apply Nomore++-like failed-literal detection\n"
-			"        no (0): Do not apply failed-literal detection")
-		("initial-lookahead", storeTo(local->heuristic().lookaheadNum)->arg("<n>"), "Restrict failed-literal detection to %A decisions\n")
-
-		("heuristic", storeTo(local->heuristic().name, &SearchOptions::parseHeuristic)->defaultsTo("Berkmin")->state(Value::value_defaulted), 
-			"Configure decision heuristic\n"
-			"      Default: %D (Unit, if --no-lookback)\n"
-			"      Valid:   Berkmin, Vmtf, Vsids, Unit, None\n"
-			"        Berkmin: Apply BerkMin-like heuristic\n"
-			"        Vmtf   : Apply Siege-like heuristic\n"
-			"        Vsids  : Apply Chaff-like heuristic\n"
-			"        Unit   : Apply Smodels-like heuristic\n"
-			"        None   : Select the first free variable")
-		("opt-heuristic" , storeTo(local->solver().strategies().heuOpts)->implicit("1")->arg("<n>"), 
-		 "Configure optimize heuristic\n"
-		  "      Valid: 0 <= <n> <= 3 / Implicit: %I \n"
-		  "         1: Apply sign heuristic\n"
-		  "         2: Apply model heuristic\n"
-		  "         3: Apply sign and model heuristic")
-		("rand-freq", notify(this, &SearchOptions::mapRandOpts)->arg("<p>"), 
-			"Make random decisions with probability %A\n"
-			"      Valid:   [0.0...1.0]\n")
-
-		("rand-prob", notify(this, &SearchOptions::mapRandOpts)->implicit("50,20")->arg("<n1,n2>"),
-			"Configure random probing\n"
-			"      Valid:   no (0), <n1,n2> (<n1> >= 0, <n2> > 0) / Implicit: %I\n"
-			"        no     : Do not run random probing\n"
-			"        <n1,n2>: Run <n1> random passes up to at most <n2> conflicts each\n")
-		("rand-watches", storeTo(local->solver().strategies().randomWatches)->implicit("true")->defaultsTo("true"),
-			"Configure watched literal initialization\n"
-			"      Default: %D\n"
-			"      Valid:   yes (1), no (0)\n"
-			"        yes: Randomly determine watched literals\n"
-			"        no : Watch first and last literal in a nogood")
-		("seed"    , storeNotify(aNumber, this, &SearchOptions::mapNumber)->arg("<n>"),    "Set random number generator's seed to %A\n")
-		("search-limit", notify(this, &SearchOptions::mapCustom)->arg("<n,m>"), "Stop search after <n> conflicts or <m> restarts\n")
-	;
-	
-	OptionGroup lookback("Clasp - Lookback Options");
-	lookback.addOptions()
-		("restarts,r", storeTo(local->solve.restart.sched, &parseSchedule)->defaultsTo("100,1.5")->state(Value::value_defaulted),
-			"Configure restart policy\n"
-			"      Default: %D\n"
-			"      Valid:   [type,]<n1[,n2,n3]> (<n1> >= 0, <n2>,<n3> > 0), no\n"
-			"      [L,]<n1>          : Run Luby et al.'s sequence with unit length <n1>\n"
-			"      [*,]<n1>,<n2>     : Run geometric sequence of <n1>*(<n2>^i) conflicts\n"
-			"      +,<n1>,<n2>       : Run arithmetic sequence of <n1>+(<n2>*i) conflicts\n"
-			"      [+,]<n1>,<n2>,<n3>: Run Biere's inner-outer sequence (<n3>=outer)\n"
-			"          <n1> = 0, no  : Disable restarts")
-		("local-restarts"  , flag(local->solve.restart.local), "Enable Ryvchin et al.'s local restarts")
-		("bounded-restarts", flag(local->solve.restart.bounded), "Enable (bounded) restarts during model enumeration")
-		("reset-restarts",   flag(local->solve.restart.resetOnModel), "Reset restart strategy during model enumeration")
-		("save-progress"   , storeTo(solverOpts->saveProgress)->implicit("1")->arg("<n>"), "Enable RSat-like progress saving on backjumps > %A")
-
-		("shuffle,s", notify(this, &SearchOptions::mapCustom)->arg("<n1,n2>"),
-			"Configure shuffling after restarts\n"
-			"      Valid:   <n1,n2> (<n1> >= 0, <n2> >= 0)\n"
-			"        <n1> > 0: Shuffle problem after <n1> and re-shuffle every <n2> restarts\n"
-			"        <n1> = 0: Do not shuffle problem after restarts\n"
-			"        <n2> = 0: Do not re-shuffle problem\n")
-
-		("deletion,d", notify(this, &SearchOptions::mapCustom)->defaultsTo("3.0,1.1,3.0")->state(Value::value_defaulted), 
-			"Configure size of learnt nogood database\n"
-			"      Default: %D\n"
-			"      Valid:   <n1[,n2,n3]> (<n3> >= <n1> >= 0, <n2> >= 1.0), no\n"
-			"        <n1,n2,n3>: Store at most min(P/<n1>*(<n2>^i),P*<n3>) learnt nogoods,\n"
-			"                    P and i being initial problem size and number of restarts\n"
-			"        no        : Do not restrict learnt db size")
-		("dsched", storeTo(local->solve.reduce.sched, &parseSchedule), "Configure secondary deletion policy (see restarts)")
-		("dglue", storeTo(solverOpts->reduceGlue), "Don't delete nogoods with lbd <= x")
-		("update-lbd", flag(solverOpts->updateLbd), "Dynamically update lbds of learnt nogoods")
-		("reduce-on-restart", flag(local->solve.reduce.reduceOnRestart), "Delete some learnt nogoods after every restart\n")
-		("estimate", flag(local->solve.reduce.estimate), "Use estimated problem complexity to init learnt db")
-		("dinit",notify(this, &SearchOptions::mapCustom)->arg("<min,max>"), "Limit initial learnt db size to range [min,max]\n")
-
-		("strengthen", storeTo(solverOpts->cflMinAntes, &mapEnum<anteTypes>)->defaultsTo("yes")->state(Value::value_defaulted),
-			"Configure conflict nogood strengthening\n"
-			"      Default: %D\n"
-			"      Valid:   bin, tern, yes (1), no (0)\n"
-			"        bin : Check only binary antecedents for self-subsumption\n"
-			"        tern: Check binary and ternary antecedents for self-subsumption\n"
-			"        yes : Check all antecedents for self-subsumption\n"
-			"        no  : Do not check antecedents for self-subsumption")
-		("recursive-str", flag(solverOpts->strengthenRecursive), "Enable MiniSAT-like conflict nogood strengthening")
-		("otfs",storeTo(solverOpts->otfs)->implicit("1"), "Enable on-the-fly subsumption (1=partial, 2=full)")
-		("reverse-arcs",storeTo(solverOpts->reverseArcs), "Enable ManySAT-like reverse-arcs\n")
-		
-		("loops", storeTo(local->loopRep, &mapEnum<loopTypes>)->defaultsTo("common")->state(Value::value_defaulted),
-			"Configure representation/learning of loop formulas\n"
-			"      Default: %D\n"
-			"      Valid:   common, distinct, shared, no\n"
-			"        common  : Create loop nogoods for atoms in an unfounded set\n"
-			"        distinct: Create distinct loop nogood for each atom in an unfounded set\n"
-			"        shared  : Create loop formula for a whole unfounded set\n"
-			"        no      : Do not learn loop formulas")
-		("loops-in-heu", storeTo(local->heuristic().loops), "Consider loop nogoods in heuristics\n")
-
-		("contraction", storeNotify(aNumber, this, &SearchOptions::mapNumber)->arg("<n>")->defaultsTo("250")->state(Value::value_defaulted),
-			"Configure temporary contraction of learnt nogoods\n"
-			"      Default: %D\n"
-			"      Valid:\n"
-			"        0  : Do not contract learnt nogoods\n"
-			"        > 0: Contract learnt nogoods containing more than %A literals\n")
-	;
-	ProgramOptions::OptionGroup hidden("Hidden Options");
-	hidden.hidden(true);
-	hidden.addOptions()
-		("berk-max", storeTo(local->heuristic().extra.berkMax), "Consider at most <n> nogoods in Berkmin heuristic")
-		("berk-moms", flag(local->heuristic().berkMoms), "Enable/Disable MOMs in Berkmin")
-		("berk-huang",flag(local->heuristic().berkHuang), "Enable/Disable Huang-scoring in Berkmin")
-		("berk-once",flag(local->heuristic().berkOnce), "Score sets (instead of multisets) in Berkmin")
-		("vmtf-mtf",storeTo(local->heuristic().extra.vmtfMtf), "In Vmtf move <n> conflict-literals to the front")
-		("nant",flag(local->heuristic().nant), "In Unit count only atoms in NAnt(P)\n")
-		("dfrac", storeNotify(aNumber, this, &SearchOptions::mapNumber), "Set fraction of nogoods to delete on reduction")
-		("dalgo", storeTo(solverOpts->reduceAlgo), "Use {0=basic, 1=sort-in-place, 2=sorted} deletion")
-		("dscore", storeTo(solverOpts->reduceScore), "Use {0=activity, 1=lbd, 2=combined} nogood scores\n")
-	;
-	root.add(search);
-	root.add(lookback);
-	root.add(hidden);
-}
-const EnumMap SearchOptions::loopTypes[] = {
-	{"common", DefaultUnfoundedCheck::common_reason}, {"shared", DefaultUnfoundedCheck::shared_reason},
-	{"distinct", DefaultUnfoundedCheck::distinct_reason}, {"no", DefaultUnfoundedCheck::only_reason},
-	{0,0}
-};
-const EnumMap SearchOptions::lookTypes[] = {
-	{"atom", Lookahead::atom_lookahead}, {"no", Lookahead::no_lookahead},
-	{"body", Lookahead::body_lookahead}, {"hybrid", Lookahead::hybrid_lookahead},
-	{0,0}
-};
-const EnumMap SearchOptions::anteTypes[] = {
-	{"all", SolverStrategies::all_antes}, {"no", SolverStrategies::no_antes},
-	{"bin", SolverStrategies::binary_antes}, {"tern", SolverStrategies::binary_ternary_antes},
-	{0,0}
-};
-
-bool SearchOptions::parseHeuristic(const std::string& s, std::string& out) {
-	std::string temp = toLower(s);
-	if      (temp == "berkmin")   { out = temp; return true; }
-	else if (temp == "vmtf")      { out = temp; return true; }
-	else if (temp == "vsids")     { out = temp; return true; }
-	else if (temp == "unit")      { out = temp; return true; }
-	else if (temp == "none")      { out = temp; return true; }
-	return false;
-}
-
-bool SearchOptions::mapRandOpts(SearchOptions* this_, const std::string& opt, const std::string& value) {
-	std::pair<double, double> v(0, 0);
-	if (value == "no" || parse(value, v)) {
-		return (opt == "rand-freq" && this_->solveOpts->setRandomProbability(v.first))
-		  ||   (opt == "rand-prob" && this_->solveOpts->setRandomizeParams((uint32)v.first, (uint32)v.second));
-	}
-	return false;
-}
-
-bool SearchOptions::parseSchedule(const std::string& s, ScheduleStrategy& sched) {
-	return parseValue(StringSlice(s), sched, 0).ok();
-}
-
-bool SearchOptions::mapNumber(SearchOptions* this_, const std::string& opt, const double* v) {
-	if      (opt == "seed")        { this_->solverOpts->rng.srand(static_cast<uint32>(*v)); }
-	else if (opt == "contraction") { this_->solverOpts->setCompressionStrategy(static_cast<uint32>(*v)); }
-	else if (opt == "dfrac")       { this_->solveOpts->reduce.setReduceFraction(*v); }
-	this_->aNumber = 0.0;
-	return true;
-}
-
-bool SearchOptions::mapCustom(SearchOptions* this_, const std::string& n, const std::string& value) {
-	bool ok = false;
-	if (n == "no-lookback") {
-		bool b;
-		ok    = FlagStr::store_true(value, b);
-		if (ok && b) this_->solverOpts->search = Clasp::SolverStrategies::no_learning;
-	}
-	else if (n == "deletion") {
-		std::vector<double> v;
-		if (value == "no" || parse(value, v)) {
-			ok = true;
-			if (v.size()==1) { v.push_back(1.1); }
-			if (v.size()==2) { v.push_back(3.0); }
-			if (v.empty())   { this_->solveOpts->reduce.disable = true; }
-			else             { this_->solveOpts->reduce.setStrategy(v[0], v[1], v[2]); }
-		}
-	}
-	else {
-		std::pair<int, int> p(-1, -1);
-		if ((ok = parse(value, p))) {
-			if (n == "dinit") {
-				this_->solveOpts->reduce.setInit((uint32)p.first, (uint32)p.second);
-			}
-			else if (n == "shuffle") {
-				this_->solveOpts->setShuffleParams(p.first, p.second);
-			}
-			else if (n == "search-limit") {
-				this_->local->solve.limits = SolveLimits(static_cast<uint64>(p.first), static_cast<uint64>(p.second));
-			}
-		}
-	}
-	return ok;
-}
-
-bool SearchOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Messages& m) {
-	if (solverOpts->search == Clasp::SolverStrategies::no_learning) {	
-		if (vm.count("heuristic") == 0) { local->heuristic().name = "unit"; }
-		if (vm.count("lookahead") == 0) { local->heuristic().lookahead = Lookahead::atom_lookahead; }
-		bool warn = local->solve.restart.local || local->solve.restart.bounded || local->solve.reduce.reduceOnRestart;
-		if (warn || vm.count("restarts") || vm.count("deletion") || vm.count("rand-prob") || vm.count("shuffle")) {
-			m.warning.push_back("lookback-options ignored because lookback strategy is not used!");     
-		}
-	}
-	return true;
-}
-
-/////////////////////////////////////////////////////////////////////////////////////////
-// clasp option validation
-/////////////////////////////////////////////////////////////////////////////////////////
-void ClaspOptions::setConfig(ClaspConfig* config) {
-	assert(config && "clasp options: config must not be 0!\n");
-	mode.config   = config;
-	search.local  = config->master();
-	this->config  = config;      
-	search.solverOpts = &search.local->solver().strategies();
-	search.solveOpts  = &search.local->solve;
-}
-void ClaspOptions::initOptions(ProgramOptions::OptionContext& root) {
-	if (mode.config == 0) {
-		throw std::runtime_error("clasp options: config not set!");
-	}
-	mode.initOptions(root);
-	search.initOptions(root);
-	basic.initOptions(root);
-}
+#else
+void ClaspOptions::initThreadOptions(ProgramOptions::OptionContext&) {}
+bool ClaspOptions::mapThreadOpts(ClaspOptions*, const std::string&, const std::string&) { return false; }
+#endif
 
 bool ClaspOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Messages& m) {
-	if (mode.validateOptions(vm, m) && search.validateOptions(vm, m)) {
+	bool ok = mode->validateOptions(vm, m) && search->validateOptions(vm, m);
+	mode.reset(0);
+	search.reset(0);
+	if (ok) {
+		satPreDefault    = vm.count("sat-prepro") == 0;
 		const char* port = 0; std::string mem;
-		if (!config->thread.portfolio.empty()) {
-			if (config->thread.portfolio == "default") {
+		if (!portfolio.empty()) {
+			if (portfolio == "default") {
 				port = portfolio_g;
 			}
 			else {
-				std::ifstream test(config->thread.portfolio.c_str());
+				std::ifstream test(portfolio.c_str());
 				if (!test) {
 					m.error = "Could not open portfolio file '";
-					m.error += config->thread.portfolio;
+					m.error += portfolio;
 					m.error += "'";
 					return false;
 				}
@@ -595,13 +614,12 @@ bool ClaspOptions::validateOptions(const ProgramOptions::ParsedOptions& vm, Mess
 			}
 		}
 		populateThreadConfigs(port, vm, m);
-		return config->validate(m.error);
 	}
-	return false;
+	return ok && config->validate(m.error);
 }
 
 void ClaspOptions::applyDefaults(Input::Format f) {
-	if (f != Input::SMODELS && mode.satPreDefault) {
+	if (f != Input::SMODELS && satPreDefault) {
 		SatElite::SatElite* pre = new SatElite::SatElite();
 		pre->options.maxIters = 20;
 		pre->options.maxOcc   = 25;
@@ -669,10 +687,7 @@ bool ClaspOptions::parseConfig(const char* in, uint32 id, const ProgramOptions::
 	}
 	try {
 		ProgramOptions::OptionContext root;
-		SearchOptions opts;
-		opts.local     = config->threadConfig(id);
-		opts.solveOpts = &opts.local->solve;
-		opts.solverOpts= &opts.local->solver().strategies();
+		SearchOptions opts(config->threadConfig(id));
 		opts.initOptions(root);
 		// 1. all from command-line
 		if (id != 0) { opts.local->initFrom(*config->master()); }
@@ -707,5 +722,6 @@ const char* portfolio_g = {
 	 "\0"/* 8 */"[SIMPLE]:  --heu=VMTF --str=no --contr=0 --restarts=100,1.3 --dinit=800,10000"
 	 ""
 };
+#undef STATE_DEFAULTED
 
 }
