@@ -244,7 +244,7 @@ void DefaultOutput::reportProgress(Enumerator::ProgressType t, const Solver& s,
 					 format_[cat_comment], format_[cat_comment], format_[cat_comment], format_[cat_comment]);
 		header_ = 20;
 	}
-	comment(3, "[%u:%c]|%7u/%-7u|%8u/%-8u|%10"PRIu64"/%-6.3f|%8"PRId64"/%-8u|\n"
+	comment(3, "[%u:%c]|%7u/%-7u|%8u/%-8u|%10"PRIu64"/%-6.3f|%8"PRId64"/%-8"PRId64"|\n"
 		, s.id()
 		, p[t]
 		, s.numFreeVars()
@@ -254,7 +254,7 @@ void DefaultOutput::reportProgress(Enumerator::ProgressType t, const Solver& s,
 		, s.stats.conflicts
 		, s.stats.conflicts/std::max(1.0,double(s.stats.choices))
 		, maxCfl <= (UINT32_MAX) ? (int64)maxCfl:-1
-		, maxL);
+		, maxL   != (UINT32_MAX) ? (int64)maxL:-1);
 }
 
 void DefaultOutput::reportState(int state, bool enter, double time) {
@@ -620,7 +620,7 @@ void JsonOutput::reportState(int state, bool enter, double) {
 
 void JsonOutput::startModel() {
 	if (!hasWitness_) {
-		startObject("Witnesseses", type_array);
+		startObject("Witnesses", type_array);
 		hasWitness_ = true;
 	}
 	if (hasModel_) {
