@@ -26,7 +26,7 @@
 #endif
 
 #if !defined(CLASP_VERSION)
-#define CLASP_VERSION "2.0.4"
+#define CLASP_VERSION "2.0.5"
 #endif
 #if !defined(CLASP_LEGAL)
 #define CLASP_LEGAL \
@@ -68,26 +68,26 @@ namespace Clasp {
 /////////////////////////////////////////////////////////////////////////////////////////
 // Parameter configuration
 /////////////////////////////////////////////////////////////////////////////////////////
-//! Options that control the decision heuristic
+//! Options that control the decision heuristic.
 struct HeuristicOptions {
 	HeuristicOptions();
 	typedef Lookahead::Type LookaheadType;
 	DecisionHeuristic* createHeuristic() const;
-	std::string   name;        /**< name of decision heuristic */
-	LookaheadType lookahead;   /**< type of lookahead */
-	int           lookaheadNum;/**< number of times lookahead is used */
-	int           loops;       /**< consider loops in heuristic (0: no, 1: yes, -1: let heuristic decide) */
+	std::string   name;        /**< Name of decision heuristic. */
+	LookaheadType lookahead;   /**< Type of lookahead. */
+	int           lookaheadNum;/**< Number of times lookahead should be used. */
+	int           loops;       /**< Consider loops in heuristic (0: no, 1: yes, -1: let heuristic decide). */
 	union Extra {
-	int      berkMax;          /**< only for Berkmin */
-	int      vmtfMtf;          /**< only for Vmtf    */
+	int      berkMax;          /**< Consider at most berkMax learnt constraints (Only for Berkmin). */
+	int      vmtfMtf;          /**< Move up to vmtfMtf conflict literals to the front (Only for Vmtf).    */
 	} extra;
-	bool     berkMoms;         /**< only for Berkmin */
-	bool     berkHuang;	       /**< only for Berkmin */
-	bool     berkOnce;         /**< only for Berkmin */
-	bool     nant;             /**< only for unit    */
+	bool     berkMoms;         /**< Only for Berkmin. */
+	bool     berkHuang;	       /**< Only for Berkmin. */
+	bool     berkOnce;         /**< Only for Berkmin. */
+	bool     nant;             /**< Only for unit.    */
 };
 
-//! Local options holding options for one solver instance
+//! Local options holding options for one solver instance.
 class LocalOptions {
 public:
 	typedef DefaultUnfoundedCheck::ReasonStrategy LoopMode;
@@ -99,8 +99,8 @@ public:
 	void              initFrom(const LocalOptions& other);
 	void              applyHeuristic();
 	Solver&           solver() { return *solver_; }
-	SolveParams       solve;   /**< strategies used during solving */ 
-	LoopMode          loopRep; /**< how to represent loops? */
+	SolveParams       solve;   /**< Strategies used during solving. */ 
+	LoopMode          loopRep; /**< How to represent loops? */
 private:
 	LocalOptions(const LocalOptions&);
 	LocalOptions& operator=(const LocalOptions&);
@@ -108,30 +108,28 @@ private:
 	HeuristicOptions* heuristic_;
 };
 
-//! Options for configuring parallel (multi-threaded) solving
+//! Options for configuring parallel (multi-threaded) solving.
 struct ThreadOptions {  
-	ThreadOptions() : forceGP(false), genTemplate(false) {}
-	struct Distribution { /**< Nogood distribution options */
+	ThreadOptions() : forceGP(false) {}
+	struct Distribution { /**< Nogood distribution options. */
 		Distribution();
 		enum Filter { filter_no = 0, filter_gp = 1, filter_sat = 2, filter_heuristic = 3 };
-		uint32 grace;       /**< lower bound on number of shared nogoods to keep */
-		uint8  lbd;         /**< upper bound on lbd for sharing nogoods */
-		uint8  types;       /**< types of nogoods to share */
-		uint8  filter;      /**< filter for integrating shared nogoods */
-		bool   copyProblem; /**< copy problem - default=false: share problem */
-	} distribute;         /**< nogood distribution parameters */
-	struct GRestarts {    /**< Options for configuring global restarts */
+		uint32 grace;       /**< Lower bound on number of shared nogoods to keep. */
+		uint8  lbd;         /**< Upper bound on lbd for sharing nogoods. */
+		uint8  types;       /**< Types of nogoods to share. */
+		uint8  filter;      /**< Filter for integrating shared nogoods (one of Filter). */
+		bool   copyProblem; /**< Copy or share initial problem constraints. */
+	} distribute;         /**< Nogood distribution parameters. */
+	struct GRestarts {    /**< Options for configuring global restarts. */
 		GRestarts():maxR(0) {}
 		uint32           maxR;
 		ScheduleStrategy sched;
-	} restarts;             /**< global restart strategy */
-	std::string portfolio;  /**< portfolio file */
-	bool        forceGP;    /**< force guiding path scheme */
-	bool        genTemplate;/**< generate a template portfolio file and exit */
+	} restarts;             /**< Global restart strategy. */
+	bool        forceGP;    /**< Force guiding path scheme. */
 };
 
 
-//! Global options controlling global solving algorithms
+//! Global options controlling global solving algorithms.
 struct GlobalOptions {
 public:
 	typedef ProgramBuilder::EqOptions        EqOptions;
@@ -142,40 +140,40 @@ public:
 	Enumerator*     initEnumerator(Enumerator::Report* r = 0);
 	bool consequences() const { return enumerate.consequences(); }
 	const char* cbType()const { return consequences() ? (enumerate.mode == enum_brave  ? "Brave" : "Cautious") : "none"; }
-	SharedContext ctx;    /**< Context-object used by all solvers */
-	ThreadOptions thread; /**< Options for parallel solving */
-	EqOptions     eq;     /**< options for equivalence preprocessing */
-	struct Optimize {     /**< Optimization options */
+	SharedContext ctx;    /**< Context-object used by all solvers. */
+	ThreadOptions thread; /**< Options for parallel solving. */
+	EqOptions     eq;     /**< Options for equivalence preprocessing. */
+	struct Optimize {     /**< Optimization options. */
 		Optimize() : hierarch(0), no(false), all(false) {}
-		WeightVec vals;     /**< initial values for optimize statements */
-		int    hierarch;    /**< use hierarchical optimization scheme */
-		bool   no;          /**< ignore optimize statements */
-		bool   all;         /**< compute all models <= vals */
+		WeightVec vals;     /**< Initial values for optimize statements. */
+		int    hierarch;    /**< Use hierarchical optimization scheme. */
+		bool   no;          /**< Ignore optimize statements. */
+		bool   all;         /**< Compute all models <= vals. */
 	}    opt;
-	struct EnumOptions {  /**< Enumeration options */
+	struct EnumOptions {  /**< Enumeration options. */
 		EnumOptions() : progress(0), numModels(-1), mode(enum_auto), projectOpts(7), project(false)
 		              , restartOnModel(false), onlyPre(false)  {}
 		bool consequences() const { return (mode & enum_consequences) != 0; }
-		Progress* progress;      /**< enable progress reporting? */
-		int       numModels;     /**< number of models to compute */
-		EnumMode  mode;          /**< enumeration type to use */
-		int       projectOpts;   /**< options for projection */
-		bool      project;       /**< enable projection */
-		bool      restartOnModel;/**< restart after each model */
-		bool      onlyPre;       /**< stop after preprocessing step? */
+		Progress* progress;      /**< Enable progress reporting? */
+		int       numModels;     /**< Number of models to compute. */
+		EnumMode  mode;          /**< Enumeration type to use. */
+		int       projectOpts;   /**< Options for projection. */
+		bool      project;       /**< Enable projection. */
+		bool      restartOnModel;/**< Restart after each model. */
+		bool      onlyPre;       /**< Stop after preprocessing step? */
 	} enumerate;
 };
 
 class ClaspFacade;
 
-//! Interface for controling incremental solving
+//! Interface for controling incremental solving.
 class IncrementalControl {
 public:
 	IncrementalControl();
 	virtual ~IncrementalControl(); 
-	//! Called before an incremental step is started
+	//! Called before an incremental step is started.
 	virtual void initStep(ClaspFacade& f)  = 0;
-	//! Called after an incremental step finished
+	//! Called after an incremental step finished.
 	/*!
 	 * \return
 	 *  - true to signal that solving should continue with next step
@@ -187,7 +185,7 @@ private:
 	IncrementalControl& operator=(const IncrementalControl&);
 };
 
-//! Parameter-object that groups & validates options
+//! Parameter-object that groups & validates options.
 class ClaspConfig : public GlobalOptions {
 public:
 	ClaspConfig();
@@ -203,48 +201,48 @@ private:
 	ClaspConfig(const ClaspConfig&);
 	ClaspConfig& operator=(const ClaspConfig&);
 	typedef PodVector<LocalOptions*>::type SolverOptions;
-	SolverOptions          solvers;
+	SolverOptions solvers;
 };
 /////////////////////////////////////////////////////////////////////////////////////////
 // ClaspFacade
 /////////////////////////////////////////////////////////////////////////////////////////
-//! Provides a simplified interface for (incrementally) solving a given problem
+//! Provides a simplified interface for (incrementally) solving a given problem.
 class ClaspFacade : public Enumerator::Report {
 public:
-	//! Defines the possible solving states
+	//! Defines the possible solving states.
 	enum State { 
-		state_start,       /*!< computation started */
-		state_read,        /*!< problem is read from input */
-		state_preprocess,  /*!< problem is prepared */
-		state_solve,       /*!< search is active */
+		state_start,       /*!< Computation started. */
+		state_read,        /*!< Problem is read from input. */
+		state_preprocess,  /*!< Problem is prepared. */
+		state_solve,       /*!< Search is active. */
 		num_states
 	};
-	//! Defines important event types
+	//! Defines important event types.
 	enum Event { 
-		event_state_enter, /*!< a new state was entered */
-		event_state_exit,  /*!< about to exit from the active state */
-		event_p_prepared,  /*!< problem was transformed to nogoods */
-		event_model        /*!< a model was found */
+		event_state_enter, /*!< A new state was entered. */
+		event_state_exit,  /*!< About to exit from the active state. */
+		event_p_prepared,  /*!< Problem was transformed to nogoods. */
+		event_model        /*!< A model was found. */
 	};
-	//! Defines possible solving results
+	//! Defines possible solving results.
 	enum Result { result_unsat, result_sat, result_unknown }; 
-	//! Callback interface for notifying about important steps in solve process
+	//! Callback interface for notifying about important steps in solve process.
 	class Callback {
 	public:
 		virtual ~Callback() {}
-		//! State transition. Called on entering/exiting a state
+		//! State transition. Called on entering/exiting a state.
 		/*!
-		 * \param e is either event_state_enter or event_state_exit
-		 * \note f.state() returns the active state
+		 * \param e Either event_state_enter or event_state_exit.
+		 * \note Call f.state() to get the active state.
 		 */
 		virtual void state(Event e, ClaspFacade& f) = 0;
-		//! Some operation triggered an important event
+		//! Some operation triggered an important event.
 		/*!
-		 * \param s the solver that triggered the event
-		 * \param e an event that is neither event_state_enter nor event_state_exit 
+		 * \param s The solver that triggered the event.
+		 * \param e An event that is neither event_state_enter nor event_state_exit.
 		 */
 		virtual void event(const Solver& s, Event e, ClaspFacade& f) = 0;
-		//! Some configuration option is unsafe/unreasonable w.r.t the current problem
+		//! Some configuration option is unsafe/unreasonable w.r.t the current problem.
 		virtual void warning(const char* msg)       = 0;
 	};
 	ClaspFacade();
@@ -262,7 +260,7 @@ public:
 	 * \pre config is valid, i.e. config.valid() returned true
 	 * \note Once solve() returned, the result of the computation can be
 	 * queried via the function result().
-	 * \note if config.onlyPre is true, solve() returns after
+	 * \note If config.onlyPre is true, solve() returns after
 	 * the preprocessing step (i.e. once the solver is prepared) and does not start a search.
 	 */
 	void solve(Input& problem, ClaspConfig& config, Callback* c);
@@ -270,7 +268,7 @@ public:
 	/*!
 	 * Incrementally solves the problem given in problem using the given configuration.
 	 * \pre config is valid, i.e. config.valid() returned true
-	 * \note Call result() to get the result of the computation
+	 * \note Call result() to get the result of the computation.
 	 * \note config.onlyPre is ignored in incremental setting!
 	 *
 	 * solveIncremental() runs a simple loop that is controlled by the
@@ -287,22 +285,22 @@ public:
 	 */
 	void solveIncremental(Input& problem, ClaspConfig& config, IncrementalControl& inc, Callback* c);
 
-	//! returns the result of a computation
+	//! Returns the result of a computation.
 	Result result() const { return result_; }
-	//! returns true if search-space was completed. Otherwise false.
+	//! Returns true if search-space was completed. Otherwise false.
 	bool   more()   const { return more_; }
-	//! returns the active state
+	//! Returns the active state.
 	State  state()  const { return state_; }
-	//! returns the current incremental step (starts at 0)
+	//! Returns the current incremental step (starts at 0).
 	int    step()   const { return step_; }
-	//! returns the current input problem
+	//! Returns the current input problem.
 	Input* input() const { return input_; }
-	//! tries to terminate an active search
+	//! Tries to terminate an active search.
 	bool   terminate() const {  return ctrl_ && ctrl_->terminate(); }
 	
 	const ClaspConfig* config() const { return config_; }
 
-	//! returns the ProgramBuilder-object that was used to transform a logic program into nogoods
+	//! Returns the ProgramBuilder-object that was used to transform a logic program into nogoods.
 	/*!
 	 * \note A ProgramBuilder-object is only created if input()->format() == Input::SMODELS
 	 * \note The ProgramBuilder-object is destroyed after the event
