@@ -307,7 +307,6 @@ public:
 	 * \pre the symbol table does not yet contain a symbol with the given id
 	 */
 	symbol_type& addUnique(key_type id, const char* name) {
-		assert(validIndirect());
 		assert((lastSort_ == 0 || map_[lastSort_-1].first < id) && "Symbol table: Invalid id in incremental step!");
 		map_.push_back(value_type(id, symbol_type(negLit(0), dupName(name))));
 		return map_.back().second;
@@ -347,7 +346,6 @@ public:
 	 * \pre no symbol was added since last call to sortUnique
 	 */
 	const_iterator lower_bound(const_iterator start, key_type i) const {
-		assert(lastSort_ == map_.size() && (start == map_.begin() || start == map_.end() || start->first <= i));
 		return std::lower_bound(start, map_.end(), i, LessKey());
 	}
 private:
@@ -370,7 +368,6 @@ private:
 		}
 		return true;
 	}
-	bool validIndirect() const { return type_ == map_indirect && lastSort_ != map_type::size_type(-1); }
 	struct LessKey {
 		bool operator()(const value_type& lhs, const value_type& rhs) const { return lhs.first < rhs.first; }
 		bool operator()(const value_type& lhs, key_type i) const { return lhs.first < i; }
