@@ -28,7 +28,6 @@
 #include <clasp/constraint.h>
 #include <clasp/util/left_right_sequence.h>
 #include <clasp/util/misc_types.h>
-
 /*!
  * \file 
  * Contains some types shared between different solvers
@@ -136,23 +135,29 @@ private:
 	InfoVec info_;
 };
 
-//! A class for efficiently storing and propagating binary and ternary clauses 
+//! A class for efficiently storing and propagating binary and ternary clauses.
 class ShortImplicationsGraph {
 public:
 	ShortImplicationsGraph();
 	~ShortImplicationsGraph();
 	//! Makes room for nodes number of nodes.
 	void resize(uint32 nodes);
+	//! Mark the instance as shared/unshared.
+	/*!
+	 * A shared instance adds learnt binary/ternary clauses
+	 * to specialized shared blocks of memory.
+	 */
+	void markShared(bool b) { shared_ = b; }
 	//! Adds the binary constraint (p, q) to the implication graph.
 	/*!
 	 * \return true iff a new implication was added.
 	 */
-	bool addBinary(Literal p, Literal q, bool learnt, bool shared);
+	bool addBinary(Literal p, Literal q, bool learnt);
 	//! Adds the ternary constraint (p, q, r) to the implication graph.
 	/*!
 	 * \return true iff a new implication was added.
 	 */
-	bool addTernary(Literal p, Literal q, Literal r, bool learnt, bool shared);
+	bool addTernary(Literal p, Literal q, Literal r, bool learnt);
 	
 	//! Removes p and its implications.
 	/*!
@@ -202,6 +207,7 @@ private:
 	ImpLists   graph_;     // one implication list for each literal
 	uint32     bin_[2];    // number of binary constraints (0: problem / 1: learnt)
 	uint32     tern_[2];   // number of ternary constraints(0: problem / 1: learnt)
+	bool       shared_;
 };
 
 //! Base class for distributing learnt knowledge between solvers.
@@ -270,6 +276,13 @@ public:
 		distributor_.reset(d); 
 	}
 
+	//! Enable/disable learning of binary/ternary clauses via ShortImplicationsGraph.
+	void enableUpdateShortImplications(bool b) {
+		updateShort_ = b;
+	}
+	//! Returns whether dynamic updates of ShortImplicationsGraph are allowed. 
+	bool updateShortImplications() const { return !frozen_ || updateShort_; }
+
 	//! Resets this object to the state after default construction.
 	void reset();
 	
@@ -420,10 +433,10 @@ public:
 	bool addUnary(Literal p);
 
 	//! Adds the binary constraint (p, q) to the shared implication graph.
-	void addBinary(Literal p, Literal q) { btig_.addBinary(p, q, false, !unique()); }
+	void addBinary(Literal p, Literal q) { btig_.addBinary(p, q, false); }
 	
 	//! Adds the ternary constraint (p, q, r) to the shared implication graph.
-	void addTernary(Literal p, Literal q, Literal r) { btig_.addTernary(p, q, r, false, !unique()); }
+	void addTernary(Literal p, Literal q, Literal r) { btig_.addTernary(p, q, r, false); }
 
 	//! Adds p as post propagator to the master solver.
 	/*!
@@ -494,23 +507,23 @@ public:
 	 */
 	//@{
 
-	//! Learns the binary clause (p, q).
-	/*!
-	 * \note 
-	 *   The binary clause (p, q) is stored in a shared data-structure but
-	 *   threads are not informed about the new clause.
-	 *   It is the caller's responsibility to distribute the new information.
+	//! Learns the unary clause (p) and distributes it to attached threads.
+	/*! 
+	 * \pre s.isTrue(p)
 	 */
-	bool learnBinary(Literal p, Literal q) { return btig_.addBinary(p, q, true, !unique()); }
+	bool learnUnary(Solver& s, Literal p, ConstraintType t);
+
+	//! Learns the binary clause (p, q) and distributes it to attached threads.
+	/*!
+	 * \return true if the clause was added.
+	 */
+	bool learnBinary(Solver& s, Literal p, Literal q, const ClauseInfo& info);
 	
-	//! Learns the ternary clause (p, q, r).
+	//! Learns the ternary clause (p, q, r) and distributes it to attached threads.
 	/*!
-	 * \note 
-	 *   The ternary clause (p, q, r) is stored in a shared data-structure but
-	 *   threads are not informed about the new clause.
-	 *   It is the caller's responsibility to distribute the new information.
+	 * \return true if the clause was added.
 	 */
-	bool learnTernary(Literal p, Literal q, Literal r) { return btig_.addTernary(p, q, r, true, !unique()); }
+	bool learnTernary(Solver& s, Literal p, Literal q, Literal r, const ClauseInfo& info);
 	
 	//! Distributes the clause in lits to other threads.
 	/*!
@@ -551,7 +564,7 @@ public:
 	SymbolTable& symTab()    const   { return symTabPtr_->symTab; }
 	void         simplifyBtig(Literal p) { btig_.removeTrue(*master(), p); }
 	const BTIG&  shortImplications() const { return btig_; }
-	uint32       numShortImplications(Literal p) const { return btig_.numEdges(p); }
+	uint32       numShortImplications(Literal p) const { return btig_.numEdges(p); }	
 	SatPrepro    satPrepro;  // preprocessor
 private:
 	SharedContext(const SharedContext&);
@@ -574,6 +587,8 @@ private:
 	size_type    lastTopLevel_;// size of master's top-level after last init
 	uint32       shareCount_;  // number of objects sharing this object
 	bool         shareConstr_; // sharing of problem constraints enabled?
+	bool         updateShort_; // allow update of BTIG after endInit()?
+	bool         frozen_;
 };
 
 //@}
