@@ -44,19 +44,19 @@ class SharedLiterals;
 ///////////////////////////////////////////////////////////////////////////////
 // Statistics
 ///////////////////////////////////////////////////////////////////////////////
-//! A struct for aggregating statistics relevant for parallel solving
+//! A struct for aggregating statistics relevant for parallel solving.
 /*!
- * Always associated with one solver (thread)
+ * Always associated with one solver (thread).
  */
 struct ParallelStats {
 	ParallelStats() { reset(); }
-	double  cpuTime;  /**< (Estimated) cpu time of the current solver */
-	uint64  splits;   /**< Number of split requests handled */
-	uint64  gps;      /**< Number of guiding paths received */
-	uint64  gpLits;   /**< Sum of literals in received guiding paths */
-	uint64  shared;   /**< Number of nogoods shared */
-	uint64  imps;     /**< Number of initial implications from shared */
-	uint64  jumps;    /**< Sum of backjumps needed to integrate new implications */
+	double  cpuTime;  /**< (Estimated) cpu time of the current solver. */
+	uint64  splits;   /**< Number of split requests handled. */
+	uint64  gps;      /**< Number of guiding paths received. */
+	uint64  gpLits;   /**< Sum of literals in received guiding paths. */
+	uint64  shared;   /**< Number of nogoods shared. */
+	uint64  imps;     /**< Number of initial implications from shared. */
+	uint64  jumps;    /**< Sum of backjumps needed to integrate new implications. */
 	bool    terminated;
 	void    reset() { 
 		std::memset(this, 0, sizeof(*this)); 
@@ -79,7 +79,7 @@ struct ParallelStats {
 	}
 };
 
-//! A struct for jump statistics
+//! A struct for jump statistics.
 struct JumpStats {
 	JumpStats() { reset(); }
 	void reset(){ std::memset(this, 0, sizeof(*this)); }
@@ -106,16 +106,16 @@ struct JumpStats {
 	}
 	double avgJumpLen()   const { return jumpSum/std::max(1.0,(double)jumps); }
 	double avgJumpLenEx() const { return (jumpSum-boundSum)/std::max(1.0,(double)jumps); }
-	uint64  jumps;    /**< Number of backjumps (i.e. number of analyzed conflicts) */
-	uint64  bJumps;   /**< Number of backjumps that were bounded */
-	uint64  jumpSum;  /**< Number of levels that could be skipped w.r.t first-uip */
-	uint64  boundSum; /**< Number of levels that could not be skipped because of backtrack-level*/
-	uint32  maxJump;  /**< Longest possible backjump */
-	uint32  maxJumpEx;/**< Longest executed backjump (< maxJump if longest jump was bounded) */
-	uint32  maxBound; /**< Max difference between uip- and backtrack-level */
+	uint64  jumps;    /**< Number of backjumps (i.e. number of analyzed conflicts). */
+	uint64  bJumps;   /**< Number of backjumps that were bounded. */
+	uint64  jumpSum;  /**< Number of levels that could be skipped w.r.t first-uip. */
+	uint64  boundSum; /**< Number of levels that could not be skipped because of backtrack-level.*/
+	uint32  maxJump;  /**< Longest possible backjump. */
+	uint32  maxJumpEx;/**< Longest executed backjump (< maxJump if longest jump was bounded). */
+	uint32  maxBound; /**< Max difference between uip- and backtrack-level. */
 };
 
-//! A struct for holding core solving statistics used by a solver
+//! A struct for holding core solving statistics used by a solver.
 struct CoreStats {
 	CoreStats() { reset(); }
 	void reset(){ std::memset(this, 0, sizeof(*this)); }
@@ -156,20 +156,20 @@ struct CoreStats {
 		--models;
 		modLits -= size;
 	}
-	uint64 choices;   /**< Number of choices performed */
-	uint64 conflicts; /**< Number of conflicts found */
-	uint64 restarts;  /**< Number of restarts */ 
-	uint64 models;    /**< Number of models found */
+	uint64 choices;   /**< Number of choices performed. */
+	uint64 conflicts; /**< Number of conflicts found. */
+	uint64 restarts;  /**< Number of restarts. */ 
+	uint64 models;    /**< Number of models found. */
 
-	uint64 learnts[Constraint_t::max_value]; /**< Number of learnt nogoods of type t-1 */
-	uint64 lits[Constraint_t::max_value];    /**< Sum of literals in nogoods of type t-1 */
-	uint64 binary;    /**< Number of learnt binary nogoods */
-	uint64 ternary;   /**< Number of learnt ternary nogoods*/
-	uint64 deleted;   /**< Sum of learnt nogoods removed */
-	uint64 modLits;   /**< Sum of decision literals in models */
+	uint64 learnts[Constraint_t::max_value]; /**< Number of learnt nogoods of type t-1. */
+	uint64 lits[Constraint_t::max_value];    /**< Sum of literals in nogoods of type t-1. */
+	uint64 binary;    /**< Number of learnt binary nogoods. */
+	uint64 ternary;   /**< Number of learnt ternary nogoods.*/
+	uint64 deleted;   /**< Sum of learnt nogoods removed. */
+	uint64 modLits;   /**< Sum of decision literals in models. */
 };
 
-//! A struct for aggregating statistics of one solve operation
+//! A struct for aggregating statistics of one solve operation.
 struct SolveStats : public CoreStats {
 	SolveStats() : jumps(0), parallel(0) { }
 	SolveStats(const SolveStats& o) : CoreStats(o), jumps(0), parallel(0) { 
@@ -193,8 +193,8 @@ struct SolveStats : public CoreStats {
 		if (!jumps) return;
 		jumps->update(dl, uipLevel, bLevel);
 	}
-	JumpStats*     jumps;    /**< optional jump statistics */
-	ParallelStats* parallel; /**< optional parallel statistics */
+	JumpStats*     jumps;    /**< Optional jump statistics. */
+	ParallelStats* parallel; /**< Optional parallel statistics. */
 private:
 	SolveStats& operator=(const SolveStats&);
 };
@@ -204,12 +204,12 @@ private:
 ///////////////////////////////////////////////////////////////////////////////
 // Clauses
 ///////////////////////////////////////////////////////////////////////////////
-//! Type for passing additional clause information 
+//! Type for passing additional clause information.
 class ClauseInfo {
 public:
 	enum { MAX_LBD = (1<<7)-1, MAX_ACTIVITY = (1<<15)-1 }; 
-	ClauseInfo() 
-		: act_(0), lbd_(MAX_LBD), rootLbd_(MAX_LBD), type_(0), tag_(0) {}
+	ClauseInfo(ConstraintType t = Constraint_t::static_constraint) 
+		: act_(0), lbd_(MAX_LBD), rootLbd_(MAX_LBD), type_(t), tag_(0) {}
 	ConstraintType type()     const { return static_cast<ConstraintType>(type_); }
 	uint32         activity() const { return static_cast<uint32>(act_); }
 	uint32         lbd()      const { return static_cast<uint32>(lbd_); }
@@ -241,7 +241,7 @@ private:
 	uint32 tag_     :  1; // conditional clause?
 };
 
-//! (Abstract) base class for clause types
+//! (Abstract) base class for clause types.
 /*!
  * ClauseHead is used to enforce a common memory-layout for all clauses.
  * It contains the two watched literals and a cache literal to improve
@@ -251,43 +251,43 @@ private:
  */
 class ClauseHead : public LearntConstraint {
 public:
-	enum { HEAD_LITS = 3, MAX_LBD = (1<<5)-1, TAGGED_CLAUSE = 1023};
+	enum { HEAD_LITS = 3, MAX_SHORT_LEN = 5, MAX_LBD = (1<<5)-1, TAGGED_CLAUSE = 1023};
 	explicit ClauseHead(const ClauseInfo& init);
 	// base interface
-	//! propagates the head and calls propagateTail() if necessary
+	//! Propagates the head and calls updateWatch() if necessary.
 	PropResult propagate(Solver& s, Literal, uint32& data);
-	//! type of clause
+	//! Type of clause.
 	ConstraintType type() const    { return ConstraintType(info_.data.type); }
-	//! true if this clause currently is the antecedent of an assignment
+	//! True if this clause currently is the antecedent of an assignment.
 	bool     locked(const Solver& s) const;
-	//! returns the activity of this clause
+	//! Returns the activity of this clause.
 	Activity activity() const       { return makeActivity(info_.data.act, info_.data.lbd); }
-	//! halves the activity of this clause
+	//! Halves the activity of this clause.
 	void     decreaseActivity()     { info_.data.act >>= 1; }
-	//! downcast from LearntConstraint
+	//! Downcast from LearntConstraint.
 	ClauseHead* clause()           { return this; }
 	
 	// clause interface
 	typedef std::pair<bool, bool> BoolPair;
-	//! increase activity
+	//! Increases activity.
 	void bumpActivity()     { info_.data.act += (info_.data.act != ClauseInfo::MAX_ACTIVITY); }
-	//! adds watches for first two literals in head to solver
+	//! Adds watches for first two literals in head to solver.
 	void attach(Solver& s);
-	//! returns true if head is satisfied w.r.t current assignment in s
+	//! Returns true if head is satisfied w.r.t current assignment in s.
 	bool satisfied(const Solver& s);
-	//! conditional clause?
+	//! Conditional clause?
 	bool tagged() const     { return info_.data.key == uint32(TAGGED_CLAUSE); }
 	uint32 lbd()  const     { return info_.data.lbd; }
 	void setLbd(uint32 lbd) { info_.data.lbd = std::min(lbd, (uint32)MAX_LBD); }
-	//! removes watches from s
+	//! Removes watches from s.
 	virtual void     detach(Solver& s);
-	//! returns the size of this clause
+	//! Returns the size of this clause.
 	virtual uint32   size()              const = 0;
-	//! returns the literals of this clause in out
+	//! Returns the literals of this clause in out.
 	virtual void     toLits(LitVec& out) const = 0;
-	//! returns true if this clause is a valid "reverse antecedent" for p
+	//! Returns true if this clause is a valid "reverse antecedent" for p.
 	virtual bool     isReverseReason(const Solver& s, Literal p, uint32 maxL, uint32 maxN) = 0;
-	//! removes p from clause if possible
+	//! Removes p from clause if possible.
 	/*!
 	 * \return
 	 *   The first component of the returned pair specifies whether or not
@@ -300,29 +300,31 @@ protected:
 	friend struct ClauseWatch;
 	bool         toImplication(Solver& s);
 	void         clearTagged(){ info_.data.key = 0; }
-	//! shall replace the watched literal at position pos with a non-false literal
+	//! Shall replace the watched literal at position pos with a non-false literal.
 	/*!
 	 * \pre pos in [0,1] 
 	 * \pre s.isFalse(head_[pos]) && s.isFalse(head_[2])
 	 * \pre head_[pos^1] is the other watched literal
 	 */
 	virtual bool updateWatch(Solver& s, uint32 pos) = 0;
-	union TailInfo {
-		struct TInfo {
-			uint32 sizeExt_; // or literal if small
-			uint32 idx_;     // or literal if small
-			void     init(bool small)  { sizeExt_ = idx_ = (small ? negLit(0).asUint() : 1); }
-			bool     isSmall()   const { return (sizeExt_ & 1u) == 0;  }
-			bool     contracted()const { return (sizeExt_ & 3u) == 3u; }
-			uint32   size()      const { return sizeExt_ >> 2; }
-			uint32   idx()       const { return idx_; }
-			void     setSize(uint32 size) { sizeExt_ = (size << 2) | (sizeExt_ & 3u); }
-			void     setIdx(uint32  idx)  { idx_ = idx; }
-			void     markContracted()     { sizeExt_ |= 2u;  }
-			void     clearContracted()    { sizeExt_ &= ~2u; }
-		}                   data;
-		SharedLiterals*   shared;
-	}       tail_;
+	union Data {
+		SharedLiterals* shared;
+		struct LocalClause {
+			uint32 sizeExt;
+			uint32 idx;
+			void   init(uint32 size) {
+				if (size <= ClauseHead::MAX_SHORT_LEN){ sizeExt = idx = negLit(0).asUint(); }
+				else                                  { sizeExt = (size << 2) + 1; idx = 0; }
+			}
+			bool   isSmall()   const    { return (sizeExt & 1u) == 0;  }
+			bool   contracted()const    { return (sizeExt & 3u) == 3u; }
+			uint32 size()      const    { return sizeExt >> 2; }
+			void   setSize(uint32 size) { sizeExt = (size << 2) | (sizeExt & 3u); }
+			void   markContracted()     { sizeExt |= 2u;  }
+			void   clearContracted()    { sizeExt &= ~2u; }
+		}               local;
+		uint32          lits[2];
+	}       data_;   // additional data
 	union Info { 
 		Info() : rep(0) {}
 		explicit Info(const ClauseInfo& i);
@@ -336,7 +338,7 @@ protected:
 	}       info_;
 	Literal head_[HEAD_LITS]; // two watched literals and one cache literal
 };
-//! Allocator for small (at most 32-byte) clauses
+//! Allocator for small (at most 32-byte) clauses.
 class SmallClauseAlloc {
 public:
 	SmallClauseAlloc();
@@ -376,7 +378,7 @@ private:
 ///////////////////////////////////////////////////////////////////////////////
 //! Represents a clause watch in a Solver.
 struct ClauseWatch {
-	//! clause watch: clause head
+	//! Clause watch: clause head
 	explicit ClauseWatch(ClauseHead* a_head) : head(a_head) { }
 	ClauseHead* head;
 	struct EqHead {
@@ -388,13 +390,13 @@ struct ClauseWatch {
 
 //! Represents a generic watch in a Solver.
 struct GenericWatch {
-	//! a constraint and some associated data
+	//! A constraint and some associated data.
 	explicit GenericWatch(Constraint* a_con, uint32 a_data = 0) : con(a_con), data(a_data) {}
-	//! calls propagate on the stored constraint and passes the stored data to that constraint.
+	//! Calls propagate on the stored constraint and passes the stored data to that constraint.
 	Constraint::PropResult propagate(Solver& s, Literal p) { return con->propagate(s, p, data); }
 	
-	Constraint* con;    /**< The constraint watching a certain literal */
-	uint32      data;   /**< Additional data associated with this watch - passed to constraint on update */
+	Constraint* con;    /**< The constraint watching a certain literal. */
+	uint32      data;   /**< Additional data associated with this watch - passed to constraint on update. */
 
 	struct EqConstraint {
 		explicit EqConstraint(Constraint* c) : con(c) {}
@@ -403,7 +405,7 @@ struct GenericWatch {
 	};	
 };
 
-//! Watch list type
+//! Watch list type.
 typedef bk_lib::left_right_sequence<ClauseWatch, GenericWatch, 0> WatchList;
 inline void releaseVec(WatchList& w) { w.clear(true); }
 
@@ -412,9 +414,9 @@ inline void releaseVec(WatchList& w) { w.clear(true); }
 ///////////////////////////////////////////////////////////////////////////////
 
 ///////////////////////////////////////////////////////////////////////////////
-//! Type for storing reasons for variable assignments together with additional data
+//! Type for storing reasons for variable assignments together with additional data.
 /*!
- * \note On 32-bit systems additional data is stored in the high-word of antecedents 
+ * \note On 32-bit systems additional data is stored in the high-word of antecedents.
  */
 struct ReasonStore32 : PodVector<Antecedent>::type {
 	uint32  dataSize() const     { return (uint32)size(); }
@@ -437,7 +439,7 @@ struct ReasonStore32 : PodVector<Antecedent>::type {
 	};
 };
 
-//! Type for storing reasons for variable assignments together with additional data
+//! Type for storing reasons for variable assignments together with additional data.
 /*
  * \note On 64-bit systems additional data is stored in a separate container.
  */
@@ -482,53 +484,53 @@ public:
 	Literal           qPop()         { return trail[front++]; }
 	void              qReset()       { front  = trail.size(); }
 
-	//! number of variables in the three-valued assignment
+	//! Number of variables in the three-valued assignment.
 	uint32            numVars()    const { return (uint32)assign_.size(); }
-	//! number of assigned variables
+	//! Number of assigned variables.
 	uint32            assigned()   const { return (uint32)trail.size();   }
-	//! number of free variables
+	//! Number of free variables.
 	uint32            free()       const { return numVars() - (assigned()+eliminated_);   }
-	//! returns the largest possible decision level
+	//! Returns the largest possible decision level.
 	uint32            maxLevel()   const { return (1u<<28)-1; }
-	//! returns v's value in the three-valued assignment
+	//! Returns v's value in the three-valued assignment.
 	ValueRep          value(Var v) const { return ValueRep(assign_[v] & 3u); }
-	//! returns v's previously saved value in the three-valued assignment
+	//! Returns v's previously saved value in the three-valued assignment.
 	ValueRep          saved(Var v) const { return v < saved_.size() ? saved_[v] : value_free; }
-	//! returns the decision level on which v was assigned if value(v) != value_free
+	//! Returns the decision level on which v was assigned if value(v) != value_free.
 	uint32            level(Var v) const { return assign_[v] >> 4u; }
-	//! returns the reason for v being assigned if value(v) != value_free
+	//! Returns the reason for v being assigned if value(v) != value_free.
 	const Antecedent& reason(Var v)const { return reason_[v]; }
-	//! returns the number of allocated data slots
+	//! Returns the number of allocated data slots.
 	uint32            numData()    const { return reason_.dataSize(); }
-	//! returns the reason data associated with v
+	//! Returns the reason data associated with v.
 	uint32            data(Var v)  const { assert(v < reason_.dataSize()); return reason_.data(v); }
 
-	//! resize to nv variables
+	//! Resize to nv variables.
 	void resize(uint32 nv) {
 		assign_.resize(nv);
 		reason_.resize(nv);
 	}
-	//! adds var to assignment - initially the new var is unassigned
+	//! Adds a var to assignment - initially the new var is unassigned.
 	Var addVar() {
 		assign_.push_back(0);
 		reason_.push_back(0);
 		return numVars()-1;
 	}
-	//! allocates data slots for nv variables to be used for storing additional reason data
+	//! Allocates data slots for nv variables to be used for storing additional reason data.
 	void requestData(uint32 nv) {
 		reason_.dataResize(nv);
 	}
-	//! eliminate var from assignment
+	//! Eliminates var from assignment.
 	void eliminate(Var v) {
 		assert(value(v) == value_free && "Can not eliminate assigned var!\n");
 		setValue(v, value_true);
 		++eliminated_;
 	}
 
-	//! assigns p.var() on level lev to the value that makes p true and store x as reason for the assignment
+	//! Assigns p.var() on level lev to the value that makes p true and stores x as reason for the assignment.
 	/*!
 	 * \return true if the assignment is consistent. False, otherwise.
-	 * \post if true is returned, p is in trail. Otherwise, ~p is.
+	 * \post If true is returned, p is in trail. Otherwise, ~p is.
 	 */
 	bool assign(Literal p, uint32 lev, const Antecedent& x) {
 		const Var      v   = p.var();
@@ -553,28 +555,28 @@ public:
 		}
 		return val == trueValue(p);
 	}
-	//! undos all assignments in the range trail[first, last).
+	//! Undos all assignments in the range trail[first, last).
 	/*!
-	 * \param first first assignment to be undone
-	 * \param save  if true, previous assignment of a var is saved before it is undone
+	 * \param first First assignment to be undone.
+	 * \param save  If true, previous assignment of a var is saved before it is undone.
 	 */
 	void undoTrail(LitVec::size_type first, bool save) {
 		if (!save) { popUntil<&Assignment::clearValue>(trail[first]); }
 		else       { saved_.resize(numVars(), 0); popUntil<&Assignment::saveAndClear>(trail[first]); }
 		front  = trail.size();
 	}
-	//! undos the last assignment
+	//! Undos the last assignment.
 	void undoLast() { clearValue(trail.back().var()); trail.pop_back(); }
-	//! returns the last assignment as a true literal
+	//! Returns the last assignment as a true literal.
 	Literal last() const { return trail.back(); }
 	Literal&last()       { return trail.back(); }
-	//! sets val as "previous value" of v
+	//! Sets val as "previous value" of v.
 	void setSavedValue(Var v, ValueRep val) {
 		if (saved_.size() <= v) saved_.resize(v+1, 0);
 		saved_[v] = val;
 	}
 	/*!
-	 * \name implementation functions
+	 * \name Implementation functions
 	 * Low-level implementation functions. Use with care and only if you
 	 * know what you are doing!
 	 */
@@ -621,6 +623,33 @@ struct ImpliedLiteral {
 	uint32      level;  /**< The earliest decision level on which lit is implied */
 	AnteInfo    ante;   /**< The reason why lit is implied on decision-level level */
 };
+//! A type for storing ImpliedLiteral objects.
+struct ImpliedList {
+	typedef PodVector<ImpliedLiteral>::type VecType;
+	typedef VecType::const_iterator iterator;
+	ImpliedList() : level(0), front(0) {}
+	//! Searches for an entry <p> in list. Returns 0 if none is found.
+	ImpliedLiteral* find(Literal p) {
+		for (uint32 i = 0, end = lits.size(); i != end; ++i) {
+			if (lits[i].lit == p)  { return &lits[i]; }
+		}
+		return 0;
+	}
+	//! Adds a new object to the list.
+	void add(uint32 dl, const ImpliedLiteral& n) {
+		if (dl > level) { level = dl; }
+		lits.push_back(n);
+	}
+	//! Returns true if list contains entries that must be reassigned on current dl.
+	bool active(uint32 dl) const { return dl < level && front != lits.size(); }
+	//! Reassigns all literals that are still implied.
+	bool assign(Solver& s);
+	iterator begin() const { return lits.begin(); }
+	iterator end()   const { return lits.end();   }
+	VecType lits;  // current set of (out-of-order) implied literals
+	uint32  level; // highest dl on which lits must be reassigned
+	uint32  front; // current starting position in lits
+};
 
 struct CCMinRecursive {
 	enum State { state_open = 0, state_poison = 1, state_removable = 2 };
