@@ -104,6 +104,62 @@ SharedLiterals* SharedLiterals::share() {
 	return this;
 }
 /////////////////////////////////////////////////////////////////////////////////////////
+// ClauseCreator::Watches
+/////////////////////////////////////////////////////////////////////////////////////////
+struct ClauseCreator::Watches {
+	Watches(const Solver& s, const Literal* begin, const Literal* end, bool knownOrder) {
+		uint32 size = uint32(end - begin);
+		if (!knownOrder) { init(s, (Literal*)begin, size, lits); }
+		else             { 
+			std::memcpy(lits, begin, std::min(size, uint32(Clause::MAX_SHORT_LEN))*sizeof(Literal));
+			abstr_f = Detail::levelAbstraction(s, lits[0]);
+			abstr_s = Detail::levelAbstraction(s, lits[1]);
+		}
+		for (uint32 i = size; i < Clause::MAX_SHORT_LEN; ++i) { lits[i] = negLit(0); }
+	}
+	Watches(const Solver& s, LitVec& in, bool knownOrder) {
+		assert(in.size() > 1);
+		if (!knownOrder) { init(s, &in[0], in.size(), 0); }
+		else             { abstr_f = Detail::levelAbstraction(s, in[0]); abstr_s = Detail::levelAbstraction(s, in[1]);  }
+		lits[0] = in[0];
+		lits[1] = in[1];
+	}
+	void init(const Solver& s, Literal* in, uint32 size, Literal* out) {
+		uint32 pos = 2, fp = 0, sp = 1;
+		abstr_f    = 0, abstr_s = 0;
+		for (uint32 i = 0; i != size; ++i) {
+			Literal p      = in[i];
+			uint32 pos_p   = i;
+			uint32 abstr_p = Detail::levelAbstraction(s, p);
+			if (abstr_p > abstr_f) {
+				std::swap(abstr_p, abstr_f);
+				pos_p = fp;
+				fp    = i;
+			}
+			if (abstr_p > abstr_s) {
+				std::swap(pos_p, sp);
+				abstr_s = abstr_p;
+			}
+			if (out && i > 1) {
+				out[pos++] = in[pos_p];
+				if (pos   == Clause::MAX_SHORT_LEN) pos = 2;
+			}
+		}
+		if (out) {
+			if (size > 0) out[0] = in[fp];
+			if (size > 1) out[1] = in[sp];
+		}
+		else if (size > 2) {
+			if (sp == 0) { std::swap(in[0], in[1]); }
+			std::swap(in[0], in[fp]);
+			std::swap(in[1], in[sp]);
+		}
+	}
+	uint32  abstr_f;
+	uint32  abstr_s;
+	Literal lits[Clause::MAX_SHORT_LEN];
+};
+/////////////////////////////////////////////////////////////////////////////////////////
 // ClauseCreator
 /////////////////////////////////////////////////////////////////////////////////////////
 ClauseCreator::ClauseCreator(Solver* s) 
@@ -115,7 +171,6 @@ void ClauseCreator::init(ConstraintType t) {
 	extra_     = ClauseInfo().setType(t);
 	fwLev_     = 0;
 	swLev_     = 0;
-	asserting_ = false;
 }
 
 ClauseCreator& ClauseCreator::start(ConstraintType t) {
@@ -127,7 +182,6 @@ ClauseCreator& ClauseCreator::startAsserting(ConstraintType t, const Literal& p)
 	init(t);
 	literals_.push_back(p);
 	fwLev_     = UINT32_MAX;
-	asserting_ = true;
 	return *this;
 }
 
@@ -159,7 +213,10 @@ void ClauseCreator::simplify() {
 	LitVec::size_type i, j = 1;
 	for (i = 1; i != literals_.size(); ++i) {
 		Literal x = literals_[i];
-		if (solver_->seen(~x)) { fwLev_ = UINT32_MAX; asserting_ = false; }
+		if (solver_->seen(~x)) { 
+			fwLev_     = UINT32_MAX; 
+			continue;
+		}
 		if (!solver_->seen(x)) {
 			solver_->markSeen(x);
 			if (i != j) { literals_[j] = literals_[i]; }
@@ -180,53 +237,62 @@ void ClauseCreator::simplify() {
 	if (literals_.size() >= 2 && sw != 1) {
 		std::swap(literals_[1], literals_[sw]);
 	}
+	if (fwLev_ == UINT32_MAX) { literals_[0] = posLit(0); }
 }
 
-ClauseCreator::Status ClauseCreator::status() const {
-	if (fwLev_ > varMax) {
-		if (asserting_)           return status_unit;
-		if (fwLev_ == UINT32_MAX) return status_subsumed;
+ClauseCreator::Status ClauseCreator::status(uint32 dl, uint32 fw, uint32 sw) {
+	if (fw > varMax)  {
+		if (fw == UINT32_MAX)      { return status_subsumed; }
+		if ((fw^UINT32_MAX) >= sw) { return status_asserting;}
 		return status_sat;
 	}
-	else if (literals_.empty()) {
-		return status_empty;
-	}
-	else if (solver_->isFalse(literals_[0])) {
+	if (sw <= dl) {
+		if (fw > dl) { return status_unit; }
+		if (fw > sw) { return status_asserting; }
 		return status_conflicting;
 	}
-	else if (literals_.size()==1 || solver_->isFalse(literals_[1])) {
-		return status_unit;
-	}
 	return status_open;
 }
+ClauseCreator::Status ClauseCreator::status() const {
+	if (literals_.empty()) { return status_empty; }
+	return status(solver_->decisionLevel(), Detail::levelAbstraction(*solver_, literals_[0]), swLev_);
+}
+
+ClauseCreator::Status ClauseCreator::status(const Solver& s, const Literal* clause_begin, const Literal* clause_end) {
+	if (clause_end <= clause_begin) { return status_empty; }
+	Watches w(s, clause_begin, clause_end, false);
+	return status(s.decisionLevel(), w.abstr_f, w.abstr_s);
+}
+
+ClauseCreator::Status ClauseCreator::status(const Solver& s, const Watches& w, uint32 flags) {
+	Status x    = status(s.decisionLevel(), w.abstr_f, w.abstr_s);
+	bool notSat = (flags & clause_not_sat) != 0;
+	bool notRoot= (flags & clause_not_root_sat) != 0;
+	if (x == status_conflicting || (x == status_asserting && s.isFalse(w.lits[0]))) {
+		x = (flags & clause_not_conflict) == 0 ? status_unit : status_subsumed;
+	}
+	else if (x == status_sat && (notSat || (notRoot && s.level(w.lits[0].var()) <= s.rootLevel()))) {
+		x = status_subsumed;
+	}
+	return x;
+}
 
 ClauseCreator::Result ClauseCreator::end() {
 	assert(solver_);
 	Result ret(0, true);
 	Solver& s     = *solver_;
-	Status  stat  = status();
-	if (stat == status_subsumed || stat == status_sat) {
-		return ret;
-	}
-	if (stat == status_empty) { 
-		ret.ok = s.addUnary(negLit(0), extra_.type());
-		return ret; 
-	}
-	if (stat == status_conflicting) {
-		ret.ok = false;
-		return ret;
-	}
+	uint32  flags = clause_known_order | clause_not_sat | clause_not_conflict;
 	if (extra_.type() != Constraint_t::static_constraint && !isSentinel(s.sharedContext()->tagLiteral())) {
 		if (std::find(literals_.begin(), literals_.end(), ~s.sharedContext()->tagLiteral()) != literals_.end()) {
 			extra_.tag();
 		}
 	}
-	return ClauseCreator::create(s, literals_, 0, extra_);
+	return ClauseCreator::create(s, literals_, flags, extra_);
 }
 
-void ClauseCreator::initWatches(Solver& s, LitVec& lits, bool allFree) {
+ClauseCreator::Status ClauseCreator::initWatches(Solver& s, LitVec& lits, bool allFree) {
 	uint32 size = static_cast<uint32>(lits.size());
-	if (size < 2) return;
+	if (size < 2) return size == 1 ? status_unit : status_empty;
 	uint32 fw, sw;
 	if (allFree) {
 		if (s.strategies().randomWatches) {
@@ -258,30 +324,11 @@ void ClauseCreator::initWatches(Solver& s, LitVec& lits, bool allFree) {
 		}
 		std::swap(lits[0], lits[fw]);
 		std::swap(lits[1], lits[sw]);
+		return status_open;
 	}
 	else {
-		// first watch: free/sat or highest dl
-		// second watch: free/sat or second highest dl
-		fw = 0; sw = 1;
-		uint32 abstr_fw = Detail::levelAbstraction(s, lits[0]);
-		uint32 abstr_sw = 0;
-		for (LitVec::size_type i = 1; i != lits.size(); ++i) {
-			uint32 abstr_p = Detail::levelAbstraction(s, lits[i]);
-			if (abstr_p > abstr_fw) {
-				assert(abstr_fw >= abstr_sw);
-				sw        = fw;
-				abstr_sw  = abstr_fw;
-				fw        = i;
-				abstr_fw  = abstr_p;
-			}
-			else if (abstr_p > abstr_sw) {
-				sw       = i;
-				abstr_sw = abstr_p;
-			}
-		}
-		if (sw == 0) { std::swap(lits[0], lits[1]); }
-		std::swap(lits[0], lits[fw]);
-		std::swap(lits[1], lits[sw]);
+		Watches w(s, lits, false);
+		return ClauseCreator::status(s.decisionLevel(), w.abstr_f, w.abstr_s);
 	}
 }
 
@@ -323,104 +370,58 @@ ClauseHead* ClauseCreator::newLearntClause(Solver& s, LitVec& lits, const Clause
 	return ret;
 }
 
-ClauseCreator::Status ClauseCreator::status(const Solver& s, Literal* lits, uint32 flags) {
-	if (s.isFalse(lits[1]) && (!s.isTrue(lits[0]) || s.level(lits[1].var()) < s.level(lits[0].var()))) {
-		return status_unit; // or conflicting!
-	}
-	else if (s.isTrue(lits[0])) {
-		uint32 satLevel = s.level(lits[0].var());
-		return satLevel == 0 || ((flags & clause_not_sat) != 0) || ((flags & clause_not_root_sat) != 0 && satLevel <= s.rootLevel())
-			? status_subsumed
-			: status_sat;
-	}
-	return status_open;
-}
-
 ClauseCreator::Result ClauseCreator::create(Solver& s, LitVec& lits, uint32 flags, const ClauseInfo& extra) {
 	assert(s.decisionLevel() == 0 || extra.type() != Constraint_t::static_constraint);
 	Result ret;
-	if (extra.type() == Constraint_t::static_constraint && s.satPrepro()) {
-		ret.ok = s.satPrepro()->addClause(lits);
-		return ret;
-	}	
 	if (lits.size() > 1) {
-		Status x    = status(s, &lits[0], flags);
-		if (x != status_subsumed) {
-			s.strategies().heuristic->newConstraint(s, &lits[0], lits.size(), extra.type());
-			uint32 impL = (x == status_unit ? s.level(lits[1].var()) : UINT32_MAX);
-			if (lits.size() > 3 || extra.tagged() || (flags&clause_explicit) != 0) {
-				ret.local = (extra.type() == Constraint_t::static_constraint)
-						? newProblemClause(s, lits, extra, flags)
-						: newLearntClause(s, lits, extra, flags);
-				ret.ok = x != status_unit || s.force(lits[0], impL, Antecedent(ret.local));
-				return ret;
-			}
-			else if (lits.size() == 3) {
-				s.addTernary(lits[0], lits[1], lits[2], extra.type());
-				ret.ok = x != status_unit || s.force(lits[0], impL, Antecedent(~lits[1], ~lits[2]));
-				return ret;
-			}
-			else {
-				s.addBinary(lits[0], lits[1], extra.type());
-				ret.ok = x != status_unit || s.force(lits[0], impL, Antecedent(~lits[1]));
-				return ret;
-			}
+		Watches w(s, lits, (flags & clause_known_order) != 0);
+		Status x = status(s, w, flags);
+		if (x == status_subsumed) { 
+			ret.ok = lits.size() > 0 && s.isTrue(lits[0]);
+			return ret;
+		}
+		if (extra.type() == Constraint_t::static_constraint && s.satPrepro()) {
+			ret.ok = s.satPrepro()->addClause(lits);
+			return ret;
+		}		
+		s.strategies().heuristic->newConstraint(s, &lits[0], lits.size(), extra.type());
+		bool   unit = (x & status_unit) != 0;
+		uint32 impL = (unit ? s.level(lits[1].var()) : UINT32_MAX);
+		if (lits.size() > 3 || extra.tagged() || (flags&clause_explicit) != 0) {
+			ret.local = (extra.type() == Constraint_t::static_constraint)
+					? newProblemClause(s, lits, extra, flags)
+					: newLearntClause(s, lits, extra, flags);
+			ret.ok = !unit || s.force(lits[0], impL, Antecedent(ret.local));
+		}
+		else if (lits.size() == 3) {
+			s.addTernary(lits[0], lits[1], lits[2], extra);
+			ret.ok = !unit || s.force(lits[0], impL, Antecedent(~lits[1], ~lits[2]));
+		}
+		else {
+			s.addBinary(lits[0], lits[1], extra);
+			ret.ok = !unit || s.force(lits[0], impL, Antecedent(~lits[1]));
 		}
-		return ret;
 	}
 	else {
 		Literal imp = !lits.empty()	? lits[0] : negLit(0);
 		ret.ok = s.addUnary(imp, extra.type());
-		return ret;
 	}	
+	return ret;
 }
-struct ClauseCreator::Watches {
-	Watches() : fwLev(0), swLev(0) {
-		lits[0] = lits[1] = lits[2] = negLit(0);
-	}
-	void init(Solver& s, const Literal* it, const Literal* end, bool checkOrder) {
-		if (!checkOrder) {
-			std::memcpy(lits, it, std::min(uint32(end - it), uint32(Clause::MAX_SHORT_LEN))*sizeof(Literal));
-			return;
-		}
-		for (uint32 pos = 0; it != end; ++it) {
-			Literal p      = *it;
-			uint32 abstr_p = Detail::levelAbstraction(s, p);
-			lits[pos]      = p;
-			// watch the two literals with highest abstraction, i.e.
-			// prefer true over free over false literals
-			if (abstr_p > fwLev) {
-				std::swap(abstr_p, fwLev);
-				std::swap(lits[0], lits[pos]);
-			}
-			if (abstr_p > swLev) {
-				std::swap(lits[1], lits[pos]);
-				swLev = abstr_p;
-			}
-			if (++pos == Clause::MAX_SHORT_LEN) {
-				pos = 2;
-			}
-		}
-	}
-	uint32  fwLev;
-	uint32  swLev;
-	Literal lits[Clause::MAX_SHORT_LEN];
-};
 
 ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags, const ClauseInfo& e) {
 	assert(!s.hasConflict() && "ClauseCreator::integrate() - precondition violated!");
 	Detail::Sink shared( (modeFlags & clause_no_release) == 0 ? clause : 0);
 	// determine state of clause
 	Result  result;
-	Watches w;
-	w.init(s, clause->begin(), clause->end(), (modeFlags & clause_known_order) == 0);
-	Status  stat(status(s, w.lits, modeFlags));
+	Watches w(s, clause->begin(), clause->end(), (modeFlags & clause_known_order) != 0);
+	Status  stat( status(s, w, modeFlags) );
 	if (clause->size() > 1 && stat != status_subsumed) {
 		if (clause->size() > Clause::MAX_SHORT_LEN) {
 			result.local  = Clause::newShared(s, clause, e, w.lits, shared.clause == 0);
 			shared.clause = 0;
 		}
-		else if (clause->size() > 3 || (modeFlags & clause_explicit) != 0) {
+		else if (clause->size() > 3 || (modeFlags & clause_explicit) != 0 || !s.sharedContext()->updateShortImplications()) {
 			result.local  = Clause::newClause(s, w.lits, clause->size(), e);
 		}
 		else {
@@ -432,14 +433,13 @@ ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause
 		}
 		if ((modeFlags & clause_no_add) == 0) { s.addLearnt(result.local, clause->size()); }
 	}
-	if (stat == status_unit) {
+	if ((stat & status_unit) != 0) {
 		Antecedent ante = result.local ? Antecedent(result.local) : Antecedent(~w.lits[1], ~w.lits[2]);
 		uint32 DL       = s.decisionLevel();
 		uint32 impLevel = s.level(w.lits[1].var());
-		result.ok       = impLevel
-		                ? s.force(w.lits[0], impLevel, ante) 
-		                :(s.stats.addLearnt(1, e.type()), s.addUnary(w.lits[0], Constraint_t::static_constraint));
-		if (s.stats.parallel) {
+		result.ok       = s.force(w.lits[0], impLevel, ante);
+		if (clause->size() == 1) { s.stats.addLearnt(1, e.type()); }
+		if (s.stats.parallel)    {
 			s.stats.parallel->imps  += 1;
 			s.stats.parallel->jumps += (DL - s.decisionLevel());
 		}
@@ -449,10 +449,10 @@ ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause
 ClauseCreator::Result ClauseCreator::integrate(Solver& s, SharedLiterals* clause, uint32 modeFlags) { 
 	return integrate(s, clause, modeFlags, ClauseInfo().setType(clause->type())); 
 }
-
 /////////////////////////////////////////////////////////////////////////////////////////
 // Clause
 /////////////////////////////////////////////////////////////////////////////////////////
+
 void* Clause::alloc(Solver& s, uint32 lits) {
 	if (lits <= Clause::MAX_SHORT_LEN) {
 		return s.allocSmall();
@@ -485,50 +485,41 @@ ClauseHead* Clause::newShortClause(Solver& s, void* mem, const Literal* lits, ui
 Clause::Clause(Solver& s, const ClauseInfo& e, const Literal* a_lits, uint32 size, uint32 tail, bool extend) 
 	: ClauseHead(e) {
 	assert(tail == size || s.isFalse(a_lits[tail]));
-	if (size > MAX_SHORT_LEN) {
-		tail_.data.init(false);
+	data_.local.init(size);
+	if (!isSmall()) {
 		// copy literals
 		std::memcpy(head_, a_lits, size*sizeof(Literal));
-		tail           = std::max(tail, (uint32)ClauseHead::HEAD_LITS);
-		tail_.data.setSize(tail);
-		tail_.data.setIdx(3);         // next idx to look at propagation
-		if (tail < size) {            // contracted clause
-			head_[size-1].watch();      // mark last literal of clause
+		tail = std::max(tail, (uint32)ClauseHead::HEAD_LITS);
+		if (tail < size) {       // contracted clause
+			head_[size-1].watch(); // mark last literal of clause
 			Literal t = head_[tail];
 			if (s.level(t.var()) > 0) {
-				tail_.data.markContracted();
+				data_.local.markContracted();
 				if (extend) {
 					s.addUndoWatch(s.level(t.var()), this);
 				}
 			}
+			data_.local.setSize(tail);
 		}
 	}
 	else {
-		tail_.data.init(true);
 		std::memcpy(head_, a_lits, std::min(size, (uint32)ClauseHead::HEAD_LITS)*sizeof(Literal));
-		Literal* t = smallLits();
-		t[0] = size > ClauseHead::HEAD_LITS   ? a_lits[ClauseHead::HEAD_LITS]   : negLit(0);
-		t[1] = size > ClauseHead::HEAD_LITS+1 ? a_lits[ClauseHead::HEAD_LITS+1] : negLit(0);
-		assert(tail_.data.isSmall() && Clause::size() == size);
+		data_.lits[0] = size > ClauseHead::HEAD_LITS   ? a_lits[ClauseHead::HEAD_LITS].asUint()   : negLit(0).asUint();
+		data_.lits[1] = size > ClauseHead::HEAD_LITS+1 ? a_lits[ClauseHead::HEAD_LITS+1].asUint() : negLit(0).asUint();
+		assert(isSmall() && Clause::size() == size);
 	}
 	attach(s);
 }
 
 Clause::Clause(Solver& s, const Clause& other) : ClauseHead(ClauseInfo()) {
-	info_.rep      = info_.rep;
+	info_.rep      = other.info_.rep;
 	uint32 oSize   = other.size();
-	if (oSize <= MAX_SHORT_LEN) {
-		tail_.data.init(true);
+	data_.local.init(oSize);
+	if      (!isSmall())      { std::memcpy(head_, other.head_, oSize*sizeof(Literal)); }
+	else if (other.isSmall()) { std::memcpy(data_.lits, other.data_.lits, (ClauseHead::MAX_SHORT_LEN+1)*sizeof(Literal)); }
+	else { // this is small, other is not
 		std::memcpy(head_, other.head_, ClauseHead::HEAD_LITS*sizeof(Literal));
-		LitRange tail  = const_cast<Clause&>(other).tail();
-		smallLits()[0] = tail.first[0];
-		smallLits()[1] = tail.first[1];
-	}
-	else {
-		tail_.data.init(false);
-		std::memcpy(head_, other.head_, oSize*sizeof(Literal));
-		tail_.data.setSize(oSize);
-		tail_.data.setIdx(3);
+		std::memcpy(data_.lits, other.head_+ClauseHead::HEAD_LITS, 2*sizeof(Literal));
 	}
 	attach(s);
 }
@@ -559,24 +550,25 @@ void Clause::detach(Solver& s) {
 	ClauseHead::detach(s);
 }
 
-
 bool Clause::updateWatch(Solver& s, uint32 pos) {
-	int32 idx   = -3, bound = -1;
+	uint32 idx = data_.local.idx;
 	if (!isSmall()) {
-		bound = (int32)(tail_.data.size());
-		idx   = (int32)(tail_.data.idx_);
-	}
-	for (;;) {
-		for (int32 i = idx; i < bound; ++i) {
-			if (!s.isFalse(head_[i])) {
-				std::swap(head_[pos], head_[i]);
-				if (i > 0) { tail_.data.setIdx(i+1); }
-				return true;
+		for (Literal* tailS = head_ + ClauseHead::HEAD_LITS, *end = longEnd();;) {
+			for (Literal* it  = tailS + idx; it < end; ++it) {
+				if (!s.isFalse(*it)) {
+					std::swap(*it, head_[pos]);
+					data_.local.idx = (++it - tailS);
+					return true;
+				}
 			}
+			if (idx == 0) { break; }
+			end = tailS + idx;
+			idx = 0;
 		}
-		if (idx <= 3) { break; }
-		bound = idx;
-		idx   = 3;
+	}
+	else if (!s.isFalse(Literal::fromRep(data_.lits[idx=0])) || !s.isFalse(Literal::fromRep(data_.lits[idx=1]))) {
+		std::swap(head_[pos].asUint(), data_.lits[idx]);
+		return true;
 	}
 	return false;
 }
@@ -597,33 +589,30 @@ void Clause::reason(Solver& s, Literal p, LitVec& out) {
 		}
 	}
 	if (s.strategies().updateLbd && ClauseHead::type() != Constraint_t::static_constraint) {
-		setLbd( s.computeLbd(p, &out[0]+i, &out[0]+out.size()) );
+		uint32 n = s.computeLbd(p, &out[0]+i, &out[0]+out.size());
+		if (n < info_.data.lbd) { setLbd( n ); }
 	}
 }
 
 bool Clause::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
 	ClauseHead::bumpActivity();
 	uint32 other = p == head_[0];
-	bool   ok    = s.ccMinimize(~head_[other], rec) && s.ccMinimize(~head_[2], rec);
-	if (ok) {
-		LitRange t = tail();
-		for (const Literal* r = t.first; r != t.second; ++r) {
-			if (!s.ccMinimize(~*r, rec)) return false;
-		}
-		if (contracted()) {
-			for (const Literal* r = t.second; (ok = s.ccMinimize(~*r, rec)) == true && !r->watched(); ) {
-				++r;
-			}
-		}
+	if (!s.ccMinimize(~head_[other], rec) || !s.ccMinimize(~head_[2], rec)) { return false; }
+	LitRange t = tail();
+	for (const Literal* r = t.first; r != t.second; ++r) {
+		if (!s.ccMinimize(~*r, rec)) { return false; }
 	}
-	return ok;
+	if (contracted()) {
+		do {
+			if (!s.ccMinimize(~*t.second, rec)) { return false; }
+		} while (!t.second++->watched());
+	}
+	return true;
 }
 
 bool Clause::isReverseReason(const Solver& s, Literal p, uint32 maxL, uint32 maxN) {
-	uint32 other = p == head_[0];
-	if (!isRevLit(s, head_[other], maxL) || !isRevLit(s, head_[2], maxL)) {
-		return false;
-	}
+	uint32 other   = p == head_[0];
+	if (!isRevLit(s, head_[other], maxL) || !isRevLit(s, head_[2], maxL)) { return false; }
 	uint32 notSeen = !s.seen(head_[other].var()) + !s.seen(head_[2].var());
 	LitRange t     = tail();
 	for (const Literal* r = t.first; r != t.second && notSeen <= maxN; ++r) {
@@ -643,21 +632,21 @@ bool Clause::simplify(Solver& s, bool reinit) {
 		detach(s);
 		return true;
 	}
-	LitRange t  = tail();
-	Literal* it = !isSmall() ? head_+2 : t.first;
-	Literal* j  = t.second;
-	for (; it != t.second && s.value(it->var()) == value_free; ++it) { }
-	if (it != t.second) {
-		for (j = it; it != t.second; ++it) {
-			if (s.isTrue(*it))  { Clause::detach(s); return true; }
-			if (!s.isFalse(*it)){ *j++ = *it; }
-		}
-		for (Literal* r = j; r != t.second; ++r) { *r = negLit(0); }
+	LitRange t = tail();
+	Literal* it= t.first - !isSmall(), *j;
+	// skip free literals
+	while (it != t.second && s.value(it->var()) == value_free) { ++it; }
+	// copy remaining free literals
+	for (j = it; it != t.second; ++it) {
+		if      (s.value(it->var()) == value_free) { *j++ = *it; }
+		else if (s.isTrue(*it)) { Clause::detach(s); return true;}
 	}
+	// replace any false lits with sentinels
+	for (Literal* r = j; r != t.second; ++r) { *r = negLit(0); }
 	if (!isSmall()) {
 		uint32 size= std::max((uint32)ClauseHead::HEAD_LITS, static_cast<uint32>(j-head_));
-		tail_.data.setSize(size);
-		tail_.data.setIdx(3);
+		data_.local.setSize(size);
+		data_.local.idx = 0;
 		if (reinit && size > 3) {
 			detach(s);
 			std::random_shuffle(head_, j, s.strategies().rng);
@@ -697,7 +686,7 @@ bool Clause::isSatisfied(const Solver& s, LitVec& freeLits) {
 
 void Clause::undoLevel(Solver& s) {
 	assert(!isSmall());
-	uint32   t = tail_.data.size();
+	uint32   t = data_.local.size();
 	Literal* r = head_+t;
 	while (!r->watched() && s.value(r->var()) == value_free) {
 		++t;
@@ -706,12 +695,12 @@ void Clause::undoLevel(Solver& s) {
 	if (r->watched() || s.level(r->var()) == 0) {
 		r->clearWatch();
 		t += !isSentinel(*r);
-		tail_.data.clearContracted();
+		data_.local.clearContracted();
 	}
 	else {
 		s.addUndoWatch(s.level(r->var()), this);
 	}
-	tail_.data.setSize(t);
+	data_.local.setSize(t);
 }
 
 void Clause::toLits(LitVec& out) const {
@@ -771,8 +760,8 @@ Literal* Clause::removeFromTail(Solver& s, Literal* it, Literal* end) {
 		*it  = *--end;
 		*end = negLit(0);
 		if (!isSmall()) { 
-			tail_.data.setSize(tail_.data.size()-1);
-			tail_.data.setIdx(3);
+			data_.local.setSize(data_.local.size()-1);
+			data_.local.idx = 0;
 		}
 	}
 	else {
@@ -785,17 +774,17 @@ Literal* Clause::removeFromTail(Solver& s, Literal* it, Literal* end) {
 			s.addUndoWatch(nLev, this);
 		}
 		if (j != end) { (j-1)->watch(); }
-		else          { tail_.data.clearContracted(); }
+		else          { data_.local.clearContracted(); }
 		end = j;
 	}
 	return end;
 }
 uint32 Clause::size() const {
-	return !isSmall() 
-		? tail_.data.size()
-		: 2 + !isSentinel(head_[2]) + !isSentinel(*smallLits()) + !isSentinel(smallLits()[1]);
+	LitRange t = const_cast<Clause&>(*this).tail();
+	return !isSentinel(head_[2])
+		? 3 + (t.second - t.first)
+		: 2;
 }
-
 /////////////////////////////////////////////////////////////////////////////////////////
 // mt::SharedLitsClause
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -807,18 +796,18 @@ ClauseHead* SharedLitsClause::newClause(Solver& s, SharedLiterals* shared_lits,
 SharedLitsClause::SharedLitsClause(Solver& s, SharedLiterals* shared_lits, const Literal* w, const ClauseInfo& e, bool addRef) 
 	: ClauseHead(e) {
 	static_assert(sizeof(SharedLitsClause) <= 32);
-	tail_.shared = addRef ? shared_lits->share() : shared_lits;
+	data_.shared = addRef ? shared_lits->share() : shared_lits;
 	std::memcpy(head_, w, std::min((uint32)ClauseHead::HEAD_LITS, shared_lits->size())*sizeof(Literal));
 	attach(s);
 }
 
 Constraint* SharedLitsClause::cloneAttach(Solver& other) {
-	return SharedLitsClause::newClause(other, tail_.shared, ClauseInfo().setType(this->type()), head_);
+	return SharedLitsClause::newClause(other, data_.shared, ClauseInfo().setType(this->type()), head_);
 }
 
 bool SharedLitsClause::updateWatch(Solver& s, uint32 pos) {
 	Literal  other = head_[1^pos];
-	for (const Literal* r = tail_.shared->begin(), *end = tail_.shared->end(); r != end; ++r) {
+	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
 		// at this point we know that head_[2] is false so we only need to check 
 		// that we do not watch the other watched literal twice!
 		if (!s.isFalse(*r) && *r != other) {
@@ -842,18 +831,19 @@ bool SharedLitsClause::updateWatch(Solver& s, uint32 pos) {
 void SharedLitsClause::reason(Solver& s, Literal p, LitVec& out) {
 	ClauseHead::bumpActivity();
 	LitVec::size_type i = out.size();
-	for (const Literal* r = tail_.shared->begin(), *end = tail_.shared->end(); r != end; ++r) {
+	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
 		assert(s.isFalse(*r) || *r == p);
 		if (*r != p) { out.push_back(~*r); }
 	}
 	if (s.strategies().updateLbd && ClauseHead::type() != Constraint_t::static_constraint) {
-		setLbd( s.computeLbd(p, &out[0]+i, &out[0]+out.size()) );
+		uint32 n = s.computeLbd(p, &out[0]+i, &out[0]+out.size());
+		if (n < info_.data.lbd) { setLbd( n ); }
 	}
 }
 
 bool SharedLitsClause::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
 	ClauseHead::bumpActivity();
-	for (const Literal* r = tail_.shared->begin(), *end = tail_.shared->end(); r != end; ++r) {
+	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
 		if (*r != p && !s.ccMinimize(~*r, rec)) { return false; }
 	}
 	return true;
@@ -861,7 +851,7 @@ bool SharedLitsClause::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
 
 bool SharedLitsClause::isReverseReason(const Solver& s, Literal p, uint32 maxL, uint32 maxN) {
 	uint32 notSeen = 0;
-	for (const Literal* r = tail_.shared->begin(), *end = tail_.shared->end(); r != end; ++r) {
+	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
 		if (*r == p) continue;
 		if (!isRevLit(s, *r, maxL)) return false;
 		if (!s.seen(r->var()) && ++notSeen > maxN) return false;
@@ -874,7 +864,7 @@ bool SharedLitsClause::simplify(Solver& s, bool reinit) {
 		detach(s);
 		return true;
 	}
-	uint32 optSize = tail_.shared->simplify(s);
+	uint32 optSize = data_.shared->simplify(s);
 	if (optSize == 0) {
 		detach(s);
 		return true;
@@ -882,7 +872,7 @@ bool SharedLitsClause::simplify(Solver& s, bool reinit) {
 	else if (optSize <= Clause::MAX_SHORT_LEN) {
 		Literal lits[Clause::MAX_SHORT_LEN];
 		Literal* j = lits;
-		for (const Literal* r = tail_.shared->begin(), *e = tail_.shared->end(); r != e; ++r) {
+		for (const Literal* r = data_.shared->begin(), *e = data_.shared->end(); r != e; ++r) {
 			if (!s.isFalse(*r)) *j++ = *r;
 		}
 		uint32 rep= info_.rep;
@@ -897,7 +887,7 @@ bool SharedLitsClause::simplify(Solver& s, bool reinit) {
 	}
 	else if (s.isFalse(head_[2])) {
 		// try to replace cache lit with non-false literal
-		for (const Literal* r = tail_.shared->begin(), *e = tail_.shared->end(); r != e; ++r) {
+		for (const Literal* r = data_.shared->begin(), *e = data_.shared->end(); r != e; ++r) {
 			if (!s.isFalse(*r) && std::find(head_, head_+2, *r) == head_+2) {
 				head_[2] = *r;
 				break;
@@ -911,7 +901,7 @@ void SharedLitsClause::destroy(Solver* s, bool detachFirst) {
 	if (s && detachFirst) {
 		ClauseHead::detach(*s);
 	}
-	tail_.shared->release();
+	data_.shared->release();
 	void* mem = this;
 	this->~SharedLitsClause();
 	if (s) {
@@ -926,7 +916,7 @@ bool SharedLitsClause::isSatisfied(const Solver& s, LitVec& freeLits) {
 	}
 	Literal* head = head_;
 	ValueRep v;
-	for (const Literal* r = tail_.shared->begin(), *end = tail_.shared->end(); r != end; ++r) {
+	for (const Literal* r = data_.shared->begin(), *end = data_.shared->end(); r != end; ++r) {
 		if ( (v = s.value(r->var())) == value_free ) {
 			freeLits.push_back(*r);
 		}
@@ -939,22 +929,23 @@ bool SharedLitsClause::isSatisfied(const Solver& s, LitVec& freeLits) {
 }
 
 void SharedLitsClause::toLits(LitVec& out) const {
-	out.insert(out.end(), tail_.shared->begin(), tail_.shared->end());
+	out.insert(out.end(), data_.shared->begin(), data_.shared->end());
 }
 
 ClauseHead::BoolPair SharedLitsClause::strengthen(Solver&, Literal, bool) {
 	return BoolPair(false, false);
 }
 
-uint32 SharedLitsClause::size() const { return tail_.shared->size(); }
+uint32 SharedLitsClause::size() const { return data_.shared->size(); }
 
 } // end namespace mt
 
 /////////////////////////////////////////////////////////////////////////////////////////
 // LoopFormula
 /////////////////////////////////////////////////////////////////////////////////////////
-LoopFormula::LoopFormula(Solver& s, uint32 size, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch) {
-	activity_       = (uint32)s.stats.restarts + (size-numBodies);  
+LoopFormula::LoopFormula(Solver& s, uint32 size, Literal* bodyLits, uint32 numBodies, uint32 bodyToWatch, const ClauseInfo& info) {
+	activity_       = info.activity();
+	lbd_            = info.lbd();
 	end_            = numBodies + 2;
 	size_           = end_+1;
 	other_          = end_-1;
@@ -994,6 +985,7 @@ void LoopFormula::destroy(Solver* s, bool detach) {
 
 
 void LoopFormula::addAtom(Literal atom, Solver& s) {
+	++activity_;
 	uint32 pos = size_++;
 	assert(isSentinel(lits_[pos]));
 	lits_[pos] = atom;
@@ -1098,7 +1090,8 @@ Constraint::PropResult LoopFormula::propagate(Solver& s, Literal, uint32& data)
 
 // Body: all other bodies + active atom
 // Atom: all bodies
-void LoopFormula::reason(Solver&, Literal p, LitVec& lits) {
+void LoopFormula::reason(Solver& s, Literal p, LitVec& lits) {
+	uint32 os = lits.size();
 	// all relevant bodies
 	for (uint32 x = 1; x != end_-1; ++x) {
 		if (lits_[x] != p) {
@@ -1110,6 +1103,10 @@ void LoopFormula::reason(Solver&, Literal p, LitVec& lits) {
 		lits.push_back(~lits_[end_-1]);
 	}
 	++activity_;
+	if (s.strategies().updateLbd) {
+		uint32 n = s.computeLbd(p, &lits[0]+os, &lits[0]+lits.size());
+		if (n < lbd_) { lbd_ = n; }
+	}
 }
 
 bool LoopFormula::minimize(Solver& s, Literal p, CCMinRecursive* rec) {
