@@ -501,7 +501,7 @@ bool ParallelSolve::requestWork(Solver& s, PathPtr& out) {
 	while (!shared_->terminate()) {
 		// only clear path and stop conflict - we don't propagate() here
 		// because we would then have to handle any eventual conflicts
-		if (!s.popRootLevel(s.rootLevel(), false)) {
+		if (!s.popRootLevel(s.rootLevel())) {
 			// s has a real top-level conflict - 
 			// there is nothing we can do to continue
 			terminate(s, true);
@@ -546,7 +546,7 @@ bool ParallelSolve::requestWork(Solver& s, PathPtr& out) {
 // check if there is more to do
 void ParallelSolve::terminate(Solver& s, bool complete) {
 	if (!shared_->terminate()) {
-		if (complete && shared_->enumerator()->optimizeHierarchical() && s.popRootLevel(s.rootLevel(), false)) {
+		if (complete && shared_->enumerator()->optimizeHierarchical() && s.popRootLevel(s.rootLevel())) {
 			// Problem is unsat and hierarchical optimization is active.
 			// The active level is at its optimum, but lower-priority levels might
 			// still be non-optimal.
