@@ -88,7 +88,7 @@ StreamSource& StreamSource::operator++() {
 	return *this;
 }
 	
-bool StreamSource::parseInt( int& val) {
+bool StreamSource::parseInt64( int64& val) {
 	val = 0;
 	bool  pos = true;
 	skipWhite();
@@ -103,16 +103,27 @@ bool StreamSource::parseInt( int& val) {
 	int d;
 	while (**this >= '0' && **this <= '9') {
 		d    = **this - '0';
-		assert((val < (INT_MAX/10)
-			|| ((val==(INT_MAX/10)) && ((INT_MAX-(val*10))>=d)))
-			&& "Integer overflow while parsing");
 		val *= 10;
 		val += d;
 		++*this;
 	}
-	val = pos ? val : -val;
+	val  = pos ? val : -val;
 	return ok;
 }
+bool StreamSource::parseInt(int& val) { 
+	int64 x;
+	return parseInt64(x)
+		&&   x >= INT_MIN
+		&&   x <= INT_MAX
+		&&   (val=(int)x, true);
+}
+bool StreamSource::parseInt(int& val, int min, int max) { 
+	int64 x;
+	return parseInt64(x)
+		&&   x >= min
+		&&   x <= max
+		&&   (val=(int)x, true);
+}
 
 void StreamSource::skipWhite() {
 	while ( **this == ' ' || **this == '\t' ) {
@@ -206,7 +217,7 @@ bool LparseReader::parse(std::istream& prg, ProgramBuilder& api) {
 
 Var LparseReader::parseAtom() {
 	int r = -1;
-	if (!source_->parseInt(r) || r < 1 || r > (int)varMax) {
+	if (!source_->parseInt(r, 1, (int)varMax)) {
 		throw ReadError(source_->line(), (r == -1 ? "Atom id expected!" : "Atom out of bounds"));
 	}
 	return static_cast<Var>(r);
@@ -233,13 +244,13 @@ bool LparseReader::readRule(int rt) {
 	rule_.setType(type);
 	if ( type == BASICRULE || rt == CONSTRAINTRULE || rt == WEIGHTRULE) {
 		rule_.addHead(parseAtom());
-		if (rt == WEIGHTRULE && (!source_->parseInt(bound) || bound < 0)) {
+		if (rt == WEIGHTRULE && !source_->parseInt(bound, 0, INT_MAX)) {
 			throw ReadError(source_->line(), "Weightrule: Positive weight expected!");
 		}
 	}
 	else if (rt == CHOICERULE) {
 		int heads;
-		if (!source_->parseInt(heads) || heads < 1) {
+		if (!source_->parseInt(heads, 1, INT_MAX)) {
 			throw ReadError(source_->line(), "Choicerule: To few heads");
 		}
 		for (int i = 0; i < heads; ++i) {
@@ -249,18 +260,18 @@ bool LparseReader::readRule(int rt) {
 	else {
 		assert(rt == 6);
 		int x;
-		if (!source_->parseInt(x) || x != 0) {
+		if (!source_->parseInt(x,0,0)) {
 			throw ReadError(source_->line(), "Minimize rule: 0 expected!");
 		}
 	}
 	int lits, neg;
-	if (!source_->parseInt(lits) || lits < 0) {
+	if (!source_->parseInt(lits, 0, INT_MAX)) {
 		throw ReadError(source_->line(), "Number of body literals expected!");
 	}
-	if (!source_->parseInt(neg) || neg < 0 || neg > lits) {
+	if (!source_->parseInt(neg, 0, lits)) {
 		throw ReadError(source_->line(), "Illegal negative body size!");
 	}
-	if (rt == CONSTRAINTRULE && (!source_->parseInt(bound) || bound < 0)) {
+	if (rt == CONSTRAINTRULE && !source_->parseInt(bound, 0, INT_MAX)) {
 		throw ReadError(source_->line(), "Constraint rule: Positive bound expected!");
 	}
 	if (bound >= 0) {
@@ -276,7 +287,7 @@ bool LparseReader::readBody(uint32 lits, uint32 neg, bool readWeights) {
 	if (readWeights) {
 		for (uint32 i = 0; i < lits; ++i) {
 			int w;
-			if (!source_->parseInt(w) || w < 0) {
+			if (!source_->parseInt(w, 0, INT_MAX)) {
 				throw ReadError(source_->line(), "Weight Rule: bad or missing weight!");
 			}
 			rule_.body[i].second = w;
@@ -341,7 +352,7 @@ bool LparseReader::readComputeStatement() {
 
 bool LparseReader::readModels() {
 	int m;
-	if (!source_->parseInt(m) || m < 0) {
+	if (!source_->parseInt(m, 0, INT_MAX)) {
 		throw ReadError(source_->line(), "Number of models expected!");
 	}
 	return skipAllWhite(*source_);
@@ -407,16 +418,15 @@ bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, Obj
 	// c2: set if v occurs positively in the current clause
 	PodVector<uint8>::type  flags;
 	TempFile temp;
-	bool wcnf = false, partial = false;
-	int w     = 1;
-	int top   = 1;
+	bool  wcnf = false, partial = false;
+	wsum_t top = 1;
 	for (uint32 state = 0;ret;)  {
 		skipAllWhite(in);
 		if      (*in == 'c')       { skipLine(in); }
 		else if (*in == 0 && state){ break; }
 		else if (state == 1)       { // read clause
-			w = 1;
-			if (wcnf && (!in.parseInt(w) || w < 1)) { ERROR("wcnf: clause weight expected!"); }
+			int64 cw = 1;
+			if (wcnf && (!in.parseInt64(cw) || cw < 1)) { ERROR("wcnf: clause weight expected!"); }
 			int lit;
 			Literal rLit;
 			bool sat = false;
@@ -428,7 +438,9 @@ bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, Obj
 				rLit = lit >= 0 ? posLit(lit) : negLit(-lit);
 				if (lit == 0) {
 					nc.start();
-					if (!sat && top != w) {
+					if (!sat && top != cw) {
+						weight_t w = (weight_t)cw;
+						if (w != cw) { ERROR("Clause weight too large!"); }
 						// soft clause
 						if (currentClause.size() > 1) {
 							flags.push_back(0);
@@ -472,14 +484,14 @@ bool parseDimacsImpl(std::istream& prg, SharedContext& ctx, bool assertPure, Obj
 		}
 		else if (match(in, "p ", false)) {
 			if (match(in, "cnf", false) || (wcnf=match(in, "wcnf", false)) == true) {
-				if (!in.parseInt(numVars) || !in.parseInt(numClauses) ) {
+				if (!in.parseInt(numVars, 0, (int)varMax) || !in.parseInt(numClauses, 0, INT_MAX)) {
 					ERROR("Bad parameters in the problem line!");
 				}
-				partial = wcnf && in.parseInt(top);
+				partial = wcnf && in.parseInt64(top);
 				lastVar = numVars;
 				if (!partial) {
 					top    = 1 - (wcnf || maxSat);
-					maxVar = numVars + ((1-top)*numClauses);
+					maxVar = numVars + (int)((1-top)*numClauses);
 					flags.reserve(maxVar+1);
 					flags.resize(numVars+1);
 					ctx.reserveVars(maxVar+1);
@@ -603,7 +615,7 @@ private:
 	SharedContext&     ctx_;
 	ObjectiveFunction& obj_;
 	NLCData*           nlc_;
-	int                numVars_, products_, soft_, min_, max_, sum_;
+	int                numVars_, products_, soft_, min_, max_;
 	int                nextVar_;
 	int                maxVar_;
 	int                adjust_;
@@ -629,33 +641,35 @@ bool OPBParser::parse() {
 //  NLC-WBO: * #variable= int #constraint= int #product= int sizeproduct= int #soft= int mincost= int maxcost= int sumcost= int
 void OPBParser::parseHeader() {
 	StreamSource& in = source_;
-	numVars_ = products_ = soft_ = min_ = max_ = sum_ = 0;
+	numVars_ = products_ = soft_ = min_ = max_ = 0;
+	int64 sum= 0;
 	delete nlc_; nlc_ = 0;
+	const int MAX     = INT_MAX;
 	if (!match(in, "* #variable=", false)){ ERROR("Missing problem line \"* #variable=\""); }
 	in.skipWhite();
-	if (!in.parseInt(numVars_))           { ERROR("Number of vars expected\n"); }
+	if (!in.parseInt(numVars_,0,varMax))  { ERROR("Number of vars expected\n"); }
 	if (!match(in, "#constraint=", true)) { ERROR("Bad problem line. Missing \"#constraint=\""); }
 	int numCons;
-	if (!in.parseInt(numCons))            { ERROR("Number of constraints expected\n"); }
+	if (!in.parseInt(numCons, 0, MAX))    { ERROR("Number of constraints expected\n"); }
 	if (match(in, "#", true)) {
 		const char* next = "soft=";
 		if (match(in, "product=", true))    { 
-			if (!in.parseInt(products_))      { ERROR("Number of products expected\n"); }
+			if (!in.parseInt(products_,0,MAX)){ ERROR("Number of products expected\n"); }
 			next = "sizeproduct=";
 			if (!match(in, next,true))        { ERROR("'sizeproduct=' expected\n"); }
 			int prodLits = 0;
-			if (!in.parseInt(prodLits))       { ERROR("Number of product literals expected\n"); }
+			if (!in.parseInt(prodLits,0,MAX)) { ERROR("Number of product literals expected\n"); }
 			nlc_ = new NLCData(prodLits+products_);
 			next = "#soft=";
 		}
 		if (match(in, next, true))          {
-			if (!in.parseInt(soft_))          { ERROR("Number of soft constraints expected\n"); }
+			if (!in.parseInt(soft_,0,MAX))    { ERROR("Number of soft constraints expected\n"); }
 			if (!match(in, "mincost=", true)) { ERROR("'mincost=' expected\n"); }
 			if (!in.parseInt(min_))           { ERROR("Minimal cost expected\n"); }
 			if (!match(in, "maxcost=", true)) { ERROR("'maxcost=' expected\n"); }
 			if (!in.parseInt(max_))           { ERROR("Maximal cost expected\n"); }
 			if (!match(in, "sumcost=", true)) { ERROR("'sumcost=' expected\n"); }
-			if (!in.parseInt(sum_))           { ERROR("Sum of soft constraints cost expected\n"); }
+			if (!in.parseInt64(sum))          { ERROR("Sum of soft constraints cost expected\n"); }
 		}
 	}
 	if (!match(in, '\n', true))           { ERROR("Unrecognized characters in problem line\n"); }
@@ -686,7 +700,7 @@ void OPBParser::parseOptObjective() {
 		addConstraint(0, adjust_);
 	}
 	else if (match(in, "soft:", true)){
-		if (!in.parseInt(obj_.rhs))    { ERROR("Top cost expected!\n"); }
+		if (!in.parseInt64(obj_.rhs))  { ERROR("Top cost expected!\n"); }
 		if (obj_.rhs < 0)              { ERROR("Top cost must be positive!\n"); }
 		if (!match(in, ';', true))     { ERROR("Semicolon missing after constraint\n"); }
 		skipAllWhite(in);
@@ -701,7 +715,7 @@ void OPBParser::parseConstraint() {
 	Var            x = 0;
 	if (match(in, '[', true)) {
 		int cost = -1;
-		if (!in.parseInt(cost) || cost < 0 || !match(in, "]", true) || cost > max_ || cost < min_) {
+		if (!in.parseInt(cost,min_,max_) || !match(in, "]", true)) {
 			ERROR("Invalid soft constraint\n");
 		}
 		if (--soft_ < 0) { ERROR("Too many soft constraints\n"); }
