@@ -286,55 +286,10 @@ bool Solver::endInit() {
 }
 
 bool Solver::addUnary(Literal p, ConstraintType t) {
-	undoUntil(0);
-	const Antecedent ante(posLit(0));
-	if (decisionLevel() != 0) {
-		// remove any existing records
-		for (ImpliedLits::size_type i = 0; i < impliedLits_.size(); ++i) {
-			if (impliedLits_[i].lit == p) {
-				impliedLits_[i] = impliedLits_.back();
-				impliedLits_.pop_back();
-				--i;
-			}
-		}
-		// remember implication
-		impliedLits_.push_back(ImpliedLiteral(p, 0, ante));
-		if (isTrue(p)) {
-			// replace antecedent with stronger constraint
-			setReason(p, ante);
-		}
+	if (t != Constraint_t::static_constraint) {
+		shared_->learnUnary(*this, p, t);
 	}
-	bool ret = force(p, ante);
-	if (t != Constraint_t::static_constraint && shared_->learnBinary(p, negLit(0))) {
-		stats.addLearnt(1, t);
-		shared_->distribute(*this, p, negLit(0), negLit(0), t);	
-	}
-	return ret;
-}
-
-bool Solver::addBinary(Literal p, Literal q, ConstraintType t) {
-	assert(validWatch(~p) && validWatch(~q) && "ERROR: startAddConstraints not called!");
-	if (t == Constraint_t::static_constraint) {
-		shared_->addBinary(p, q);
-	}
-	else if (t != Constraint_t::static_constraint && shared_->learnBinary(p, q)) {
-		stats.addLearnt(2, t);
-		shared_->distribute(*this, p, q, negLit(0), t);
-	}
-	return true;
-}
-
-bool Solver::addTernary(Literal p, Literal q, Literal r, ConstraintType t) {
-	assert(validWatch(~p) && validWatch(~q) && validWatch(~r) && "ERROR: startAddConstraints not called!");
-	assert(p != q && q != r && "ERROR: ternary clause contains duplicate literal");
-	if (t == Constraint_t::static_constraint) {
-		shared_->addTernary(p, q, r);
-	}
-	else if (t != Constraint_t::static_constraint && shared_->learnTernary(p, q, r)) {
-		stats.addLearnt(3, t);
-		shared_->distribute(*this, p, q, r, t);
-	}
-	return true;
+	return force(p, 0, Antecedent(posLit(0)));
 }
 
 void Solver::add(Constraint* c) {
@@ -347,24 +302,21 @@ uint32 Solver::numConstraints() const {
 		+ (shared_ ? shared_->numBinary()+shared_->numTernary() : 0);
 }
 
-bool Solver::popRootLevel(uint32 i, bool resolve)  {
+bool Solver::popRootLevel(uint32 i)  {
 	clearStopConflict();
 	if (i > rootLevel_) i = rootLevel_;
 	rootLevel_ = btLevel_ = rootLevel_ - i;
-	// first, resolve any open conflicts
-	if (hasConflict() && resolve && !resolveConflict()) {
-		return false;
-	}
-	// second, go back to new root level and try to re-assert still implied literals
-	undoUntil(rootLevel_);
+	impliedLits_.front    = 0;
+	// go back to new root level and re-assert still implied literals
+	undoUntil(rootLevel_, true);
 	if (!isTrue(sharedContext()->tagLiteral())) {
 		removeConditional();
 	}
-	return forceImplied();
+	return !hasConflict();
 }
 
 bool Solver::clearAssumptions()  {
-	return popRootLevel(rootLevel(), false)
+	return popRootLevel(rootLevel())
 		&& simplify();
 }
 
@@ -403,9 +355,9 @@ void Solver::updateGuidingPath(LitVec& gpOut, LitVec::size_type& start, uint32&
 	}
 	start = end;
 	uint32 implLits = 0;
-	for (LitVec::size_type i = 0; i != impliedLits_.size(); ++i) {
-		if (impliedLits_[i].level <= rootLevel_ && ++implLits > implied) {
-			gpOut.push_back(impliedLits_[i].lit);
+	for (ImpliedList::iterator it = impliedLits_.begin(); it != impliedLits_.end(); ++it) {
+		if (it->level <= rootLevel_ && ++implLits > implied) {
+			gpOut.push_back(it->lit);
 		}
 	}
 	implied = implLits;
@@ -721,7 +673,7 @@ bool Solver::resolveConflict() {
 			uint32 uipLevel = analyzeConflict();
 			stats.updateJumps(decisionLevel(), uipLevel, btLevel_);
 			undoUntil( uipLevel );
-			return ClauseCreator::create(*this, cc_, 0, ccInfo_);
+			return ClauseCreator::create(*this, cc_, ccInfo_);
 		}
 		else {
 			return backtrack();
@@ -736,24 +688,26 @@ bool Solver::backtrack() {
 		if (decisionLevel() == rootLevel_) return false;
 		lastChoiceInverted = ~decision(decisionLevel());
 		btLevel_ = decisionLevel() - 1;
-		undoUntil(btLevel_);
-	} while (!forceImplied() || !force(lastChoiceInverted, 0));
+		undoUntil(btLevel_, true);
+	} while (hasConflict() || !force(lastChoiceInverted, 0));
 	return true;
 }
 
-bool Solver::forceImplied() {
-	bool r = !hasConflict();
-	const uint32 DL = decisionLevel();
-	ImpliedLits::size_type j = 0;
-	for (ImpliedLits::size_type i = 0; i < impliedLits_.size(); ++i) {
-		if (impliedLits_[i].level <= DL) {
-			r = r && force(impliedLits_[i].lit, impliedLits_[i].ante.ante(), impliedLits_[i].ante.data());
-			impliedLits_[j++] = impliedLits_[i];
+bool ImpliedList::assign(Solver& s) {
+	assert(front <= lits.size());
+	bool ok             = !s.hasConflict();
+	const uint32 DL     = s.decisionLevel();
+	VecType::iterator j = lits.begin() + front;
+	for (VecType::iterator it = j, end = lits.end(); it != end; ++it) {
+		if (it->level <= DL && ok) {
+			ok = s.force(it->lit, it->ante.ante(), it->ante.data());
 		}
+		if (it->level < DL) { *j++ = *it; }
 	}
-	if (DL == 0) j = 0;
-	impliedLits_.erase(impliedLits_.begin()+j, impliedLits_.end());
-	return r;
+	lits.erase(j, lits.end());
+	level = DL * uint32(!lits.empty());
+	front = level > s.rootLevel() ? front  : lits.size();
+	return ok;
 }
 
 void Solver::undoUntil(uint32 level) {
@@ -771,16 +725,14 @@ void Solver::undoUntil(uint32 level) {
 }
 
 uint32 Solver::undoUntil(uint32 level, bool popBt) {
-	bool checkImplied = false;
 	if (popBt && backtrackLevel() > level && !shared_->project(decision(backtrackLevel()).var())) {
 		setBacktrackLevel(level);
-		checkImplied = true;
 	}
 	undoUntil(level);
-	if (checkImplied) {
-		forceImplied();
+	if (impliedLits_.active(level = decisionLevel())) {
+		impliedLits_.assign(*this);
 	}
-	return decisionLevel();
+	return level;
 }
 
 uint32 Solver::estimateBCP(const Literal& p, int rd) const {
@@ -803,6 +755,31 @@ uint32 Solver::estimateBCP(const Literal& p, int rd) const {
 	}
 	return (uint32)i;
 }
+
+uint32 Solver::inDegree(WeightLitVec& out) {
+	if (decisionLevel() == 0) { return 1; }
+	out.reserve((numAssignedVars()-levelStart(1))/10);
+	uint32 maxIn  = 1;
+	uint32 i      = assign_.trail.size(), stop = levelStart(1);
+	for (Antecedent xAnte; i-- != stop; ) {
+		Literal x     = assign_.trail[i];
+		uint32  xLev  = assign_.level(x.var());
+		xAnte         = assign_.reason(x.var());
+		uint32  xIn   = 0;
+		if (!xAnte.isNull() && xAnte.type() != Antecedent::binary_constraint) {
+			conflict_.clear();
+			xAnte.reason(*this, x, conflict_);
+			for (LitVec::const_iterator it = conflict_.begin(); it != conflict_.end(); ++it) {
+				xIn += level(it->var()) != xLev;
+			}
+			if (xIn) {
+				out.push_back(WeightLiteral(x, xIn));
+				maxIn     = std::max(xIn, maxIn);
+			}
+		}
+	}
+	return maxIn;
+}
 /////////////////////////////////////////////////////////////////////////////////////////
 // Solver: Private helper functions
 ////////////////////////////////////////////////////////////////////////////////////////
