@@ -58,7 +58,7 @@ bool ClauseHead::satisfied(const Solver& s) {
 bool ClauseHead::toImplication(Solver& s) {
 	ConstraintType t = ClauseHead::type();
 	uint32 newSize   = isSentinel(head_[1]) ? 1 : 2 + (!s.isFalse(head_[2]) || s.level(head_[2].var()) > 0);
-	bool   upBTIG    = s.sharedContext()->unique() || t != Constraint_t::static_constraint;
+	bool   upBTIG    = s.sharedContext()->updateShortImplications() && (t != Constraint_t::static_constraint || s.sharedContext()->unique());
 	bool   locked    = ClauseHead::locked(s) && s.decisionLevel() > 0;
 	if (newSize == 1 || (!tagged() && !locked && upBTIG)) {
 		uint64 bin = 0, tern = 0, learn = 0, lits = 0;
