@@ -93,7 +93,9 @@ bool WeightConstraint::newWeightConstraint(SharedContext& ctx, Literal con, Weig
 				return false;
 			}
 		}
-		if (i == lits.size()) return true;
+		if (i == lits.size() || bound <= 0) {
+			return true;
+		}
 		lits.erase(lits.begin(), lits.begin()+i);
 		if (lits.front().second == lits.back().second && lits.front().second != 1) {
 			// disguised cardinality constraint
@@ -125,6 +127,7 @@ weight_t WeightConstraint::canonicalize(Solver& s, WeightLitVec& lits, weight_t&
 	// Step 1: remove assigned/superfluous literals and merge duplicate/complementary ones
 	LitVec::size_type j = 0, other;
 	for (LitVec::size_type i = 0; i != lits.size(); ++i) {
+		lits[i].first.clearWatch();
 		Literal x = lits[i].first;
 		if (s.value(x.var()) == value_free && lits[i].second > 0) {
 			if (!s.seen(x.var())) { // first time we see x, keep and mark x
@@ -197,17 +200,12 @@ WeightConstraint::WeightConstraint(SharedContext& ctx, Literal W, const WeightLi
 		mt::SharedWeightConstraintLits* x = new (m) mt::SharedWeightConstraintLits(uint32(1+lits.size()), hasWeights);
 		lits_ = &x->shared_lits;
 	}
-	Literal* heu  = new Literal[1+lits.size()];
-	heu[0]        = W;
 	Literal* p    = lits_->lits_;
+	Literal* h    = (Literal*)undo_;
 	Solver&  s    = *ctx.master();
 	bool needData       = lits_->hasWeights();
 	bound_[WL::FFB_BTB]	= (sumWeights-bound)+1;// ffb-btb
 	bound_[WL::FTB_BFB]	= bound;               // ftb-bfb
-	up_                 = undoStart();         // undo stack is initially empty
-	undo_[0].data       = 0;
-	undo_[up_].data     = 0;
-	setBpIndex(1);                             // where to start back propagation
 	*p++                = ~W;                  // store constraint literal
 	if (hasWeights) *p++= Literal::fromRep(1); // and weight if necessary
 	ctx.setFrozen(W.var(), true);              // exempt from variable elimination
@@ -216,7 +214,7 @@ WeightConstraint::WeightConstraint(SharedContext& ctx, Literal W, const WeightLi
 		? NOT_ACTIVE                             // both constraints are relevant
 		: WL::FFB_BTB+s.isFalse(W);              // otherwise only one is relevant
 	for (uint32 i = 0, end = (uint32)lits.size(); i != end; ++i) {
-		*p++ = heu[i+1] = lits[i].first;         // store constraint literal
+		*p++ = h[i+1] = lits[i].first;           // store constraint literal
 		if (hasWeights) {                        // followed by weight 
 			*p++= Literal::fromRep(lits[i].second);// if necessary
 		}
@@ -224,11 +222,16 @@ WeightConstraint::WeightConstraint(SharedContext& ctx, Literal W, const WeightLi
 		addWatch(s, i+1, WL::FFB_BTB);           // watches ~lits[idx]
 		ctx.setFrozen(lits[i].first.var(), true);// exempt from variable elimination
 		if (needData) ctx.requestData(lits[i].first.var());
-		undo_[up_+i+1].data = 0;
-	}	
-	// Initialize heuristic with literals (no weights) in constraint.
-	s.strategies().heuristic->newConstraint(s, heu, 1+lits.size(), Constraint_t::static_constraint);
-	delete [] heu;
+	}
+	// init heuristic
+	uint32 off = active_ != NOT_ACTIVE;
+	h[0]       = W;
+	s.strategies().heuristic->newConstraint(s, h+off, lits.size()+(1-off), Constraint_t::static_constraint);
+	// init undo stack
+	up_                 = undoStart();         // undo stack is initially empty
+	undo_[0].data       = 0;
+	undo_[up_].data     = 0;
+	setBpIndex(1);                             // where to start back propagation
 	if (active_ == NOT_ACTIVE) {
 		addWatch(s, 0, WL::FTB_BFB);             // watch con in both phases
 		addWatch(s, 0, WL::FFB_BTB);             // in order to allow for backpropagation
@@ -242,7 +245,7 @@ WeightConstraint::WeightConstraint(SharedContext& ctx, Literal W, const WeightLi
 WeightConstraint::WeightConstraint(Solver& s, const WeightConstraint& other) {
 	lits_           = other.lits_->share();
 	WL::Iterator it = lits_->begin();
-	Literal* heu    = new Literal[lits_->size()];
+	Literal* heu    = (Literal*)undo_;
 	heu[0]          = ~*it;
 	bound_[0]	      = other.bound_[0];
 	bound_[1]	      = other.bound_[1];
@@ -259,8 +262,9 @@ WeightConstraint::WeightConstraint(Solver& s, const WeightConstraint& other) {
 		}
 	}
 	// Initialize heuristic with literals (no weights) in constraint.
-	s.strategies().heuristic->newConstraint(s, heu, lits_->size(), Constraint_t::static_constraint);
-	delete [] heu;
+	uint32 off = active_ != NOT_ACTIVE;
+	s.strategies().heuristic->newConstraint(s, heu+off, lits_->size()-off, Constraint_t::static_constraint);
+	// Init undo stack
 	std::memcpy(undo_, other.undo_, sizeof(UndoInfo)*(lits_->size()+lits_->hasWeights()));
 	up_ = other.up_;
 }
