@@ -48,6 +48,7 @@ class ClauseCreatorTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testCreateNonAssertingLearntClause);
 	CPPUNIT_TEST(testCreateLearntClauseConflict);
 	CPPUNIT_TEST(testCreateNonAssertingLearntClauseAsserting);
+	CPPUNIT_TEST(testCreateBogusUnit);
 
 	CPPUNIT_TEST(testIntegrateEmpty);
 	CPPUNIT_TEST(testIntegrateUnit);
@@ -180,11 +181,7 @@ public:
 	}
 	void testCreatorSimplifyFindsTauts() {
 		creator.start().add(a).add(b).add(c).add(a).add(b).add(~a).simplify();
-		CPPUNIT_ASSERT(creator.size() == 4);
-		CPPUNIT_ASSERT(creator[0] == a);
-		CPPUNIT_ASSERT(creator[1] == b);
-		CPPUNIT_ASSERT(creator[2] == c);
-		CPPUNIT_ASSERT(creator[3] == ~a);
+		CPPUNIT_ASSERT(creator.size() <= 4);
 		creator.end();
 		CPPUNIT_ASSERT(0 == ctx.numConstraints());
 	}
@@ -260,9 +257,9 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, (bool)creator.end());
 		CPPUNIT_ASSERT_EQUAL(true, solver().isTrue(d));
 		CPPUNIT_ASSERT(solver().numLearntConstraints() == 1);
-
 		// test with a short clause
 		solver().undoUntil(0);
+		solver().reduceLearnts(1.0f);
 		solver().assume(a); solver().force(b,0); solver().propagate();// level 1
 		solver().assume(c); solver().propagate();										// level 2
 		creator.start(Constraint_t::learnt_loop);
@@ -274,6 +271,23 @@ public:
 		CPPUNIT_ASSERT_EQUAL(true, solver().isTrue(d));
 	}
 
+	void testCreateBogusUnit() {
+		solver().assume(a) && solver().propagate();
+		solver().assume(~b) && solver().propagate();
+		solver().force(~d,0) && solver().propagate();
+		solver().assume(~c) && solver().propagate();
+		CPPUNIT_ASSERT(solver().decisionLevel() == 3);
+		
+		creator.start(Constraint_t::learnt_other).add(d).add(b).add(c).add(a);
+		CPPUNIT_ASSERT(ClauseCreator::status(solver(), &creator.lits()[0], &creator.lits()[0] + creator.size()) == ClauseCreator::status_sat);
+
+		ClauseCreator::Result r = creator.end();
+		CPPUNIT_ASSERT_EQUAL(true, r.ok);
+		CPPUNIT_ASSERT(solver().decisionLevel() == 3);
+
+		creator.start(Constraint_t::learnt_other).add(a).add(~c).end();
+	}
+
 	void testIntegrateEmpty() {
 		LitVec cl;
 		solver().assume(~a) && solver().propagate();
