@@ -366,7 +366,7 @@ public:
 		clLits.push_back(b);
 		clLits.push_back(~a);
 		ClauseInfo extra; extra.tag().setType(Constraint_t::learnt_conflict);
-		ClauseHead* c = ClauseCreator::create(*solver, clLits, 0, extra).local;
+		ClauseHead* c = ClauseCreator::create(*solver, clLits, extra).local;
 		CPPUNIT_ASSERT(c->size() == 2);
 		CPPUNIT_ASSERT(solver->isTrue(b) && solver->reason(b).constraint() == c);
 		solver->backtrack() && solver->propagate();
