@@ -1239,13 +1239,13 @@ public:
 		SharedContext* other = &ctx;
 		LitVec lits; lits.resize(2);
 		lits[0] = a; lits[1] = b;
-		ClauseCreator::create(s, lits, 0, ClauseInfo().setType(Constraint_t::learnt_conflict));
+		ClauseCreator::create(s, lits, ClauseInfo().setType(Constraint_t::learnt_conflict));
 		lits.resize(3);
 		lits[0] = ~a; lits[1] = ~b; lits[2] = ~c;
-		ClauseCreator::create(s, lits,0, ClauseInfo().setType(Constraint_t::learnt_loop));
+		ClauseCreator::create(s, lits, ClauseInfo().setType(Constraint_t::learnt_loop));
 		lits.resize(1);
 		lits[0] = d;
-		ClauseCreator::create(s, lits, 0, ClauseInfo().setType(Constraint_t::learnt_conflict));
+		ClauseCreator::create(s, lits, ClauseInfo().setType(Constraint_t::learnt_conflict));
 		CPPUNIT_ASSERT(dummy->unary  == 1);
 		CPPUNIT_ASSERT(dummy->binary == 1);
 		CPPUNIT_ASSERT(dummy->ternary == 1);
